// ===========================================================================
//	PP Basic Starter.cp 		1994-1996 Metrowerks Inc. All rights reserved.
// ===========================================================================
//
//	This file contains the starter code for a basic PowerPlant application

#include "PP Basic Starter.h"

#include <LGrowZone.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>
#include <LEditField.h>

// My headers & enum
#include <LGroupBox.h>
#include "VideoSession.h"
#include "OffscreenWorld.h"
#include "OffscreenGroup.h"
#include "AsyncBuffer.h"
enum mainErr { NewCWindowErr };

// put declarations for resource ids (ResIDTs) here

const ResIDT	window_Sample		= 1;	// EXAMPLE


// ===========================================================================
//		 Main Program
// ===========================================================================

void main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.
									
	CPPStarterApp	theApp;			// replace this with your App type
	theApp.Run();
}


// ---------------------------------------------------------------------------
//		 CPPStarterApp 			// replace this with your App type
// ---------------------------------------------------------------------------
//	Constructor

CPPStarterApp::CPPStarterApp()
{
	// Register functions to create core PowerPlant classes
	RegisterAllPPClasses();
	URegistrar::RegisterClass(LGroupBox::class_ID,		(ClassCreatorFunc) LGroupBox::CreateGroupBoxStream);
	
	// Create a window
	theWindowP = LWindow::CreateWindow(window_Sample, this);
	theViewP = (LView*)theWindowP->FindPaneByID(100);
	theWindowP->Show();
	
	// Setup the member pointers
	try
	{
		theSessionP = new VideoSession( AppleVdig );
		theOffscreenRect.top = 0;
		theOffscreenRect.left = 0;
		theOffscreenRect.bottom = 120;
		theOffscreenRect.right = 160;
		theImage0P = new OffscreenWorld( 16, theOffscreenRect, nil );
		theImage1P = new OffscreenWorld( 16, theOffscreenRect, nil );
		theImage2P = new OffscreenWorld( 16, theOffscreenRect, nil );
		theImageGroupP = new OffscreenGroup();
		theImageGroupP->AddOffWorldPtr( theImage0P );
		theImageGroupP->AddOffWorldPtr( theImage1P );
		theImageGroupP->AddOffWorldPtr( theImage2P );
		theBufferP = new AsyncBuffer( theSessionP, theImageGroupP );
		
		VDGrabOneFrameAsync( theSessionP->VdigComponent(), 0 );		// bug
		VDGrabOneFrameAsync( theSessionP->VdigComponent(), 1 );
		
	}
	
	// exception handlers
	catch ( VideoErr	theErr )
	{
		SysBeep(0);
		ExitToShell();
	}
	
	catch ( OffscreenErr theErr )
	{
		SysBeep(0);
		ExitToShell();
	}
	
	catch ( AsyncBufferErr theErr )
	{
		SysBeep(0);
		ExitToShell();
	}
	
	catch ( OffscreenGroupErr theErr )
	{
		SysBeep(0);
		ExitToShell();
	}
}


// ---------------------------------------------------------------------------
//		 ~CPPStarterApp			// replace this with your App type
// ---------------------------------------------------------------------------
//	Destructor
//

CPPStarterApp::~CPPStarterApp()
{
}


// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CPPStarterApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) 
	{
		default:
			cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	This function enables menu commands.
//

void
CPPStarterApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{

	switch (inCommand) 
	{
		default:
			LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}


// ---------------------------------------------------------------------------
//		 ProcessNextEvent
// ---------------------------------------------------------------------------
//	Overriding the ProcessNextEvent member function.
//  That's not the most elegant way of doing this, but that's simple!
//

void
CPPStarterApp::ProcessNextEvent()
{
	while ( !VDDone( theSessionP->VdigComponent(), 0 ) ) {};
	VDGrabOneFrameAsync( theSessionP->VdigComponent(), 2 ); 
	theViewP->FocusDraw();
	CopyBits( (BitMap*)*theImage0P->PixMapH(), &(theViewP->GetMacPort()->portBits), &theOffscreenRect, &theOffscreenRect, srcCopy, nil );		
	LApplication::ProcessNextEvent();
			
	while ( !VDDone( theSessionP->VdigComponent(), 1 ) ) {};
	VDGrabOneFrameAsync( theSessionP->VdigComponent(), 0 ); 
	theViewP->FocusDraw();
	CopyBits( (BitMap*)*theImage1P->PixMapH(), &(theViewP->GetMacPort()->portBits), &theOffscreenRect, &theOffscreenRect, srcCopy, nil );		
	LApplication::ProcessNextEvent();
			
	while ( !VDDone( theSessionP->VdigComponent(), 2 ) ) {};
	VDGrabOneFrameAsync( theSessionP->VdigComponent(), 1 ); 
	theViewP->FocusDraw();
	CopyBits( (BitMap*)*theImage2P->PixMapH(), &(theViewP->GetMacPort()->portBits), &theOffscreenRect, &theOffscreenRect, srcCopy, nil );
    LApplication::ProcessNextEvent();
}