/*****
 *
 *	CursorUtil.c
 *
 *	This is a support file for "Grant's CGI Framework".
 *	Please see the license agreement that accompanies the distribution package
 *	for licensing details.
 *
 *	Copyright 1996 by Grant Neufeld
 *	grant@acm.org  grant@kagi.com
 *	http://arpp.carleton.ca/cgi/framework/
 *
 *****/

#include "MyConfiguration.h"

#include "compiler_stuff.h"
#include "globals.h"

#include "DebugUtil.h"


/***  LOCAL VARIABLES  ***/

static CursorHList	vCursorsBW = nil;
static CCursorHList	vCursorsColor = nil;
static SysEnvRec	theWorld;
	

/***  LOCAL PROTOTYPES  ***/

static void	InitColorCursors	( Int16 );
static void	InitBWCursors		( Int16 );
static void	SpinColorCursor		( Int16 );
static void	SpinBWCursor		( Int16 );


/***  FUNCTIONS  ***/

/*  */
void
InitCursors ( Int16 theCursResID )
{
	SysEnvirons( curSysEnvVers, &theWorld );
	if ( theWorld.hasColorQD )
	{
		InitColorCursors ( theCursResID );
	}
	else
	{
		InitBWCursors ( theCursResID );
	}
} /*  */

/* initialize color anuimated cursor structure */
void
InitColorCursors ( Int16 theCursResID )
{
	CCrsrHandle	theCursor;
	Int16		counter;
	
	vCursorsColor = (CCursorHList) GetResource ( 'acur', theCursResID );
	if ( vCursorsColor != NULL )
	{
		for ( counter = 0; counter < (**vCursorsColor).cnt; counter++ )
		{
			theCursor = (CCrsrHandle) GetCCursor ( (**vCursorsColor).curs[counter].i.i );
			HLockHi ( (Handle) theCursor );
			(**vCursorsColor).curs[counter].h = theCursor;
		}
	}
} /*  */

/* initialize black and white animated cursor list */
void
InitBWCursors ( Int16 theCursResID )
{
	CursHandle	theCursor;
	Int16		counter;
	
	vCursorsBW = (CursorHList) GetResource ( 'acur', theCursResID );
	if ( vCursorsBW != NULL )
	{
		for ( counter = nil; counter < (**vCursorsBW).cnt; counter++ )
		{
			theCursor = (CursHandle) GetCursor( (**vCursorsBW).curs[counter].i.i );
			HLockHi ( (Handle) theCursor );
			(**vCursorsBW).curs[counter].h = theCursor;
		}
	}
} /*  */

/* Changes cursor to next in the list.
	if delta is 1, the next cursor in the list will be used.
	else if delta is -1, the previous cursor in the list will be used. */
void
SpinACURCursor ( Int16 delta )
{
	if ( theWorld.hasColorQD )
	{
		SpinColorCursor ( delta );
	}
	else
	{
		SpinBWCursor ( delta );
	}
} /*  */

/* Changes color cursor to next in the list.
	if delta is 1, the next cursor in the list will be used.
	else if delta is -1, the previous cursor in the list will be used. */
void
SpinColorCursor ( Int16 delta )
{
	Int16 c;
	
	if ( vCursorsColor != nil )
	{
		c = (**vCursorsColor).current + delta;
		if ( c >= (**vCursorsColor).cnt )
		{
			c = nil;
		}
		else if ( c < nil )
		{
			c = (**vCursorsColor).cnt - 1;
		}
		
		(**vCursorsColor).current = c;
		SetCCursor( (**vCursorsColor).curs[c].h );
	}
} /*  */

/* Changes B&W cursor to next in the list.
	if delta is 1, the next cursor in the list will be used.
	else if delta is -1, the previous cursor in the list will be used. */
void SpinBWCursor(Int16 delta)
{
	Int16 c;
	
	if(vCursorsBW != nil)
	{
		c = (**vCursorsBW).current + delta;
		if( c >= (**vCursorsBW).cnt )
		{
			c = nil;
		}
		else if ( c < nil )
		{
			c = (**vCursorsBW).cnt - 1;
		}
		
		(**vCursorsBW).current = c;
		SetCursor( *((**vCursorsBW).curs[c].h) );
	}
} /*  */


/*****  EOF  *****/
