/*****
 *
 *	StringUtil.c
 *
 *	This is a support file for "Grant's CGI Framework".
 *	Please see the license agreement that accompanies the distribution package
 *	for licensing details.
 *
 *	Copyright 1995,1996 by Grant Neufeld
 *	grant@acm.com
 *	http://arpp.carleton.ca/cgi/framework/
 *
 *****/

#include <string.h>

#include "MyConfiguration.h"

#include "DebugUtil.h"

#include "StringUtil.h"


/***  FUNCTIONS  ***/

/* Copy pascal format srcStr to the destStr */
void
StringPascalCopy ( char *srcStr, char *destStr )
{
	BlockMove ( srcStr, destStr, (srcStr[nil]) + 1 );
} /* StringPascalCopy */


/* Case-insensitive comparison of two strings.
	Returns:
		nil if no difference between the strings
		<0 if str1 < str2
		>0 if str1 > str2 */
p_export
short
StringCompareNoCase ( char *str1, char *str2 )
{
	char	char1;
	char	char2;
	
	my_assert ( str1 != NULL, "\pStringCompareNoCase: str1 is NULL" );
	my_assert ( str2 != NULL, "\pStringCompareNoCase: str2 is NULL" );
	
	do
	{
		char1 = CharMakeUpper ( *str1 );
		char2 = CharMakeUpper ( *str2 );
		
		if ( char1 < char2 )
		{
			return -1;
		}
		else if ( char1 > char2 )
		{
			return 1;
		}
		else
		{
			str1++;
			str2++;
		}
	} while ( (*str1 != nil) && (*str2 != nil) );
	
	if ( *str1 != nil )
	{
		/* str1 still has chars, so is greater than str2 */
		return 1;
	}
	else if ( *str1 != nil )
	{
		/* str2 still has chars, so is greater than str1 */
		return -1;
	}
	else
	{
		/* all the characters in both strings matched */
		return 0;
	}
} /* StringCompareNoCase */


/* Case-insensitive comparison of two strings.
	Returns:
		nil if no difference between the strings
		<0 if str1 < str2
		>0 if str1 > str2 */
p_export
short
StringNCompareNoCase ( char *str1, char *str2, UInt16 theSize )
{
	char	char1;
	char	char2;
	UInt16	count;
	
	my_assert ( str1 != NULL, "\pStringNCompareNoCase: str1 is NULL" );
	my_assert ( str2 != NULL, "\pStringNCompareNoCase: str2 is NULL" );
	my_assert ( theSize != nil, "\pStringNCompareNoCase: theSize is zero" );
	
	count = nil;
	
	do
	{
		char1 = CharMakeUpper ( *str1 );
		char2 = CharMakeUpper ( *str2 );
		
		if ( char1 < char2 )
		{
			return -1;
		}
		else if ( char1 > char2 )
		{
			return 1;
		}
		else
		{
			str1++;
			str2++;
		}
		
		count++;
	} while ( (*str1 != nil) && (*str2 != nil) && (count < theSize) );
	
	/* all the characters in both strings, up to theSize, matched */
	return 0;
} /* StringNCompareNoCase */


/* convert the given character to uppercase.
	just return theChar if it is not between a and z */
char
CharMakeUpper ( char theChar )
{
	if ( (theChar >= 'a') && (theChar <= 'z') )
	{
		return theChar + ('A' - 'a');
	}
	else
	{
		return theChar;
	}
} /* CharMakeUpper */


/* count the occurances of theChar in theString */
long
StringCountChar ( char *theString, char theChar )
{
	long	total;
	char *	tempStr;
	
	total	= nil;
	tempStr	= theString;
	
	do
	{
		/* get a ptr to the next instance of theChar */
		tempStr = StringChar ( tempStr, theChar );
		
		if ( (tempStr != NULL) && (*tempStr != nil) )
		{
			tempStr++;
			total++;
		}
	} while ( (tempStr != NULL) && (*tempStr != nil) );
	
	return total;
} /* StringCountChar */


/* Returns true if any chars were converted */
Boolean
StringConvertCharToChar ( char *ioString, char fromChar, char toChar )
{
	Boolean	didConvert;
	
	didConvert = false;
	
	while ( ioString[0] != nil )
	{
		if ( ioString[0] == fromChar )
		{
			ioString[0] = toChar;
			didConvert  = true;
		}
		ioString++;
	}
	
	return didConvert;
} /* StringConvertCharToChar */


/* working version of strchr for TPM */
#if kCompiling_For_Symantec
char *
StringChar ( const char *theString, unsigned char theChar )
{
	do
	{
		if ( *theString == theChar )
		{
			/* found the character, return a pointer to it */
			return (char *)theString;
		}
		theString++;
	} while ( *theString != nil );
	
	/* didn't find the character */
	return NULL;
} /* StringChar */
#endif


/**  TEXT DRAWING FUNCTIONS  **/

/* Font, size, style, etc. must be set before this function is called. */
void
StringDrawInRect ( StringPtr theString, Rect theRect )
{
	if ( theString[nil] > nil )
	{
		StringDrawTextInRect ( (char *)(theString + 1), theString[nil], theRect );
	}
} /* StringDrawInRect */


/* returns the number of characters not drawn because of insufficient space.
	zero if every character drawn */
long
StringDrawTextInRect ( char *theText, long totalLength, Rect theRect )
{
	char *		currentStartOfText;
	long		charsLeftInText;
	long		currentNumCharsToPrint;
	FontInfo	fInfo;
	short		textLineHeight;
	short		currentVPosition;
	short		hPosition;
	StyledLineBreakCode		theBreakCode;
	Fixed		rectWidth;
	Fixed		textWidth;
	
	/* initialize text values */
	currentStartOfText		= theText;
	charsLeftInText			= totalLength;
	currentNumCharsToPrint	= charsLeftInText;
	
	GetFontInfo ( &fInfo );
	
	/* initialize spacing */
	textLineHeight		= fInfo.ascent + fInfo.descent + fInfo.leading;
	currentVPosition	= theRect.top + fInfo.ascent;
	hPosition			= theRect.left;
	rectWidth			= Long2Fix ( theRect.right - theRect.left );
	
	/* clear the rectangle where the text is to be displayed */
	EraseRect ( &theRect );
	
	/* while there are still characters in the text that haven't been drawn
		and there is still room in the Rect for drawing more characters */
	while ( (charsLeftInText > nil) && (currentVPosition < theRect.bottom) )
	{
		textWidth = rectWidth;
		
		theBreakCode = StyledLineBreak ( currentStartOfText, charsLeftInText, nil,
			charsLeftInText, nil, &textWidth, &currentNumCharsToPrint);
		
		MoveTo   ( hPosition, currentVPosition );
		DrawText ( currentStartOfText, nil, currentNumCharsToPrint );
		
		currentVPosition   += textLineHeight;
		charsLeftInText    -= currentNumCharsToPrint;
		currentStartOfText += currentNumCharsToPrint;
	} /* end of while */
	
	return charsLeftInText;
} /* StringDrawTextInRect */


/***  EOF  ***/
