///--------------------------------------------------------------------------------------
//	BlitPixie8Bit.c
//
//	Ideas and code snippets contributed by:
//		Ben Sharpe, Brigham Stevens, Sean Callahan, Joe Britt and Tim Collins
//
//	Portions are copyright:  1991-94 Tony Myles
//
//	Description: Implementation of BlitPixie8Bit - a fast 8-bit blitter
///--------------------------------------------------------------------------------------


#ifndef __SWCOMMON__
#include "SWCommonHeaders.h"
#endif

#ifndef __SPRITEWORLDUTILS__
#include "SpriteWorldUtils.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __QDOFFSCREEN__
#include <QDOffscreen.h>
#endif

#ifndef __SPRITEWORLD__
#include "SpriteWorld.h"
#endif

#ifndef __SPRITEFRAME__
#include "SpriteFrame.h"
#endif

#ifndef __BLITPIXIE__
#include "BlitPixie.h"
#endif

#ifndef __SPRITECOMPILER__
#include "SpriteCompiler.h"
#endif


extern char 				gSWmmuMode;



///--------------------------------------------------------------------------------------
//		BlitPixie8BitRectDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BlitPixie8BitRectDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect* srcRect,
	Rect* dstRect)
{
	Rect 					srcBlitRect = *srcRect;
	Rect 					dstBlitRect = *dstRect;
	
	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;

	{
		unsigned long 		numBytesPerRow;
		SInt8 				mmuMode;
	
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);

		BlitPixie8Bit(
				// calculate the address of the first byte of the source
			(PixelChunkPtr)(srcFrameP->frameBaseAddr + 
				(srcFrameP->scanLinePtrArray[srcBlitRect.top - srcFrameP->frameRect.top]) + 
				srcBlitRect.left),

				// calculate the address of the first byte of the destination
			(PixelChunkPtr)(dstFrameP->frameBaseAddr + 
				(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),

				// calculate the number of rows to blit
			dstBlitRect.bottom - dstBlitRect.top,

				// number of bytes in a row (duh!)
			numBytesPerRow,
#if SW_USE_C
				// for PPC, just frameRowBytes
			srcFrameP->frameRowBytes,
			dstFrameP->frameRowBytes);
#else
				// for 68K, offset from end of one row to beginning of next row
			srcFrameP->frameRowBytes - numBytesPerRow,
			dstFrameP->frameRowBytes - numBytesPerRow);
#endif
		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}


///--------------------------------------------------------------------------------------
//		BP8BitInterlacedRectDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BP8BitInterlacedRectDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect* srcRect,
	Rect* dstRect)
{
	Rect 		srcBlitRect = *srcRect;
	Rect 		dstBlitRect = *dstRect;

	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
		
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
		// If first line is not on an even number, then skip it.
	if ( (dstBlitRect.top - dstFrameP->frameRect.top)>>1<<1 != 
		dstBlitRect.top - dstFrameP->frameRect.top )
	{
		srcBlitRect.top++;
		dstBlitRect.top++;
	}

	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;

	{
		unsigned long 		numBytesPerRow;
		unsigned long		numRowsToCopy;
		SInt8 				mmuMode;
		
		
			// Is destBlitRect height an even number?
		numRowsToCopy = dstBlitRect.bottom - dstBlitRect.top;		
		
		if ( numRowsToCopy>>1<<1 == numRowsToCopy )
			numRowsToCopy>>=1;
		else
			numRowsToCopy = (numRowsToCopy>>1) + 1;
		
		if (numRowsToCopy < 1)
			return;
		
		
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);
		
		
		BlitPixie8Bit(
			// calculate the address of the first byte of the source
		(PixelChunkPtr)(srcFrameP->frameBaseAddr + 
			(srcFrameP->scanLinePtrArray[srcBlitRect.top - srcFrameP->frameRect.top]) + 
			srcBlitRect.left),
				
			// calculate the address of the first byte of the destination
		(PixelChunkPtr)(dstFrameP->frameBaseAddr + 
			(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),
			
			// number of rows to copy, already calculated for Interlaced mode
		numRowsToCopy,
		
			// number of bytes in a row (duh!)
		numBytesPerRow,
		
#if SW_USE_C
				// for PPC, just frameRowBytes * 2
			srcFrameP->frameRowBytes<<1,
			dstFrameP->frameRowBytes<<1);
#else
				// for 68K, offset from end of one row to beginning of next row
			((srcFrameP->frameRowBytes<<1) - numBytesPerRow),
  			((dstFrameP->frameRowBytes<<1) - numBytesPerRow));
#endif		

		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}


///--------------------------------------------------------------------------------------
//		BlitPixie8BitMaskDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BlitPixie8BitMaskDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect *srcRect,
	Rect *dstRect)
{
	Rect dstBlitRect = *dstRect;
	Rect srcBlitRect = *srcRect;
	
	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;
	
	
	{
		unsigned long 		numBytesPerRow;
		unsigned long 		srcBaseOffset;
		SInt8 				mmuMode;
				
	
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
			// calculate the offset to the first byte of the source
		srcBaseOffset = srcFrameP->scanLinePtrArray[srcBlitRect.top - 
			srcFrameP->frameRect.top] + srcBlitRect.left;

			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);

		BlitPixieMask8Bit(
				// calculate the address of the first byte of the source
			(PixelPtr)(srcFrameP->frameBaseAddr + srcBaseOffset),

				// calculate the address of the first byte of the destination
			(PixelPtr)(dstFrameP->frameBaseAddr + 
				(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),

				// calculate the address of the first byte of the mask
			(PixelPtr)(srcFrameP->maskBaseAddr + srcBaseOffset),

				// calculate the number of rows to blit
			dstBlitRect.bottom - dstBlitRect.top,

				// number of bytes in a row (duh!)
			numBytesPerRow,

#if SW_USE_C
			// for PPC, just frameRowBytes
		srcFrameP->frameRowBytes,
		dstFrameP->frameRowBytes);
#else
			// for 68K, offset from end of one row to beginning of next row
		srcFrameP->frameRowBytes - numBytesPerRow,
		dstFrameP->frameRowBytes - numBytesPerRow);
#endif	
		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}

///--------------------------------------------------------------------------------------
//		BlitPixie8BitPartialMaskDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BlitPixie8BitPartialMaskDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect *srcRect,
	Rect *dstRect)
{
	Rect dstBlitRect = *dstRect;
	Rect srcBlitRect = *srcRect;
	
	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;
	
	
	{
		unsigned long 		numBytesPerRow;
		unsigned long 		srcBaseOffset;
		SInt8 				mmuMode;
				
	
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
			// calculate the offset to the first byte of the source
		srcBaseOffset = srcFrameP->scanLinePtrArray[srcBlitRect.top - srcFrameP->frameRect.top] + 
			srcBlitRect.left;

			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);

		BlitPixiePartialMask8Bit(
				// calculate the address of the first byte of the source
			(PixelPtr)(srcFrameP->frameBaseAddr + srcBaseOffset),

				// calculate the address of the first byte of the destination
			(PixelPtr)(dstFrameP->frameBaseAddr + 
				(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),

				// calculate the address of the first byte of the mask
			(PixelPtr)(srcFrameP->maskBaseAddr + srcBaseOffset),

				// calculate the number of rows to blit
			dstBlitRect.bottom - dstBlitRect.top,

				// number of bytes in a row (duh!)
			numBytesPerRow,

#if SW_USE_C
			// for PPC, just frameRowBytes
		srcFrameP->frameRowBytes,
		dstFrameP->frameRowBytes);
#else
			// for 68K, offset from end of one row to beginning of next row
		srcFrameP->frameRowBytes - numBytesPerRow,
		dstFrameP->frameRowBytes - numBytesPerRow);
#endif	
		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}


///--------------------------------------------------------------------------------------
//		BP8BitInterlacedMaskDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BP8BitInterlacedMaskDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect* srcRect,
	Rect* dstRect)
{
	Rect 	srcBlitRect = *srcRect;
	Rect 	dstBlitRect = *dstRect;

	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
		// If first line is not on an even number, then skip it.
	if ( (dstBlitRect.top - dstFrameP->frameRect.top)>>1<<1 != 
		dstBlitRect.top - dstFrameP->frameRect.top )
	{
		srcBlitRect.top++;
		dstBlitRect.top++;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;

	{
		unsigned long 		numBytesPerRow;
		unsigned long		numRowsToCopy;
		unsigned long 		srcBaseOffset;
		SInt8 				mmuMode;
		
		
			// Is destBlitRect height an even number?
		numRowsToCopy = dstBlitRect.bottom - dstBlitRect.top;		
	
		if ( numRowsToCopy>>1<<1 == numRowsToCopy )
			numRowsToCopy>>=1;
		else
			numRowsToCopy = (numRowsToCopy>>1) + 1;
			
		if (numRowsToCopy < 1)
			return;
		
		
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
		
			// calculate the offset to the first byte of the source
		srcBaseOffset = srcFrameP->scanLinePtrArray[srcBlitRect.top - srcFrameP->frameRect.top] + 
			srcBlitRect.left;
		
			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);
		
		
		BlitPixieMask8Bit(
			// calculate the address of the first byte of the source
		(PixelPtr)(srcFrameP->frameBaseAddr + srcBaseOffset),
		
			// calculate the address of the first byte of the destination
		(PixelPtr)(dstFrameP->frameBaseAddr + 
			(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),
		
			// calculate the address of the first byte of the mask
		(PixelPtr)(srcFrameP->maskBaseAddr + srcBaseOffset),
			
			// number of rows to copy, already calculated for Interlaced mode
		numRowsToCopy,
		
			// number of bytes in a row (duh!)
		numBytesPerRow,
		
#if SW_USE_C
				// for PPC, just frameRowBytes * 2
			srcFrameP->frameRowBytes<<1,
			dstFrameP->frameRowBytes<<1);
#else
				// for 68K, offset from end of one row to beginning of next row
			((srcFrameP->frameRowBytes<<1) - numBytesPerRow),
  			((dstFrameP->frameRowBytes<<1) - numBytesPerRow));
#endif

		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}

///--------------------------------------------------------------------------------------
//		BP8BitInterlacedPartialMaskDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void BP8BitInterlacedPartialMaskDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect* srcRect,
	Rect* dstRect)
{
	Rect 	srcBlitRect = *srcRect;
	Rect 	dstBlitRect = *dstRect;

	
		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)	// can't use "else if"s here because the
	{														// deltaRects sent to SWScreenDrawProc
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;	// can have a "wrap around" effect
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
		// If first line is not on an even number, then skip it.
	if ( (dstBlitRect.top - dstFrameP->frameRect.top)>>1<<1 != 
		dstBlitRect.top - dstFrameP->frameRect.top )
	{
		srcBlitRect.top++;
		dstBlitRect.top++;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;

	{
		unsigned long 		numBytesPerRow;
		unsigned long		numRowsToCopy;
		unsigned long 		srcBaseOffset;
		SInt8 				mmuMode;
		
		
			// Is destBlitRect height an even number?
		numRowsToCopy = dstBlitRect.bottom - dstBlitRect.top;		
	
		if ( numRowsToCopy>>1<<1 == numRowsToCopy )
			numRowsToCopy>>=1;
		else
			numRowsToCopy = (numRowsToCopy>>1) + 1;
			
		if (numRowsToCopy < 1)
			return;
		
		
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
		
			// calculate the offset to the first byte of the source
		srcBaseOffset = srcFrameP->scanLinePtrArray[srcBlitRect.top - srcFrameP->frameRect.top] + 
			srcBlitRect.left;
		
			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);
		
		
		BlitPixiePartialMask8Bit(
			// calculate the address of the first byte of the source
		(PixelPtr)(srcFrameP->frameBaseAddr + srcBaseOffset),
		
			// calculate the address of the first byte of the destination
		(PixelPtr)(dstFrameP->frameBaseAddr + 
			(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),
		
			// calculate the address of the first byte of the mask
		(PixelPtr)(srcFrameP->maskBaseAddr + srcBaseOffset),
			
			// number of rows to copy, already calculated for Interlaced mode
		numRowsToCopy,
		
			// number of bytes in a row (duh!)
		numBytesPerRow,
		
#if SW_USE_C
				// for PPC, just frameRowBytes * 2
			srcFrameP->frameRowBytes<<1,
			dstFrameP->frameRowBytes<<1);
#else
				// for 68K, offset from end of one row to beginning of next row
			((srcFrameP->frameRowBytes<<1) - numBytesPerRow),
  			((dstFrameP->frameRowBytes<<1) - numBytesPerRow));
#endif

		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}


///--------------------------------------------------------------------------------------
//		CompiledSprite8BitDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void CompiledSprite8BitDrawProc(
	FramePtr srcFrameP,
	FramePtr dstFrameP,
	Rect *srcRect,
	Rect *dstRect)
{
#if !SW_PPC
	Rect 		dstBlitRect = *dstRect;
	Rect 		srcBlitRect = *srcRect;
	SInt8		mmuMode;
	

	// we first check to see if any clipping will be necessary, then
	// we determine whether or not we can use the compiled blitter. we must
	// do this since the compiled blitter cannot handle clipping.

		// clip off the top so we don't write into random memory
	if (dstBlitRect.top < dstFrameP->frameRect.top)
	{
		srcBlitRect.top += dstFrameP->frameRect.top - dstBlitRect.top;
		dstBlitRect.top =  dstFrameP->frameRect.top;
	}
	
		// clip off the bottom
	if (dstBlitRect.bottom > dstFrameP->frameRect.bottom)
	{
		srcBlitRect.bottom -= dstBlitRect.bottom - dstFrameP->frameRect.bottom;
		dstBlitRect.bottom = dstFrameP->frameRect.bottom;
	}
	
		// clip off the left
	if (dstBlitRect.left < dstFrameP->frameRect.left)
	{
		srcBlitRect.left += dstFrameP->frameRect.left - dstBlitRect.left;
		dstBlitRect.left = dstFrameP->frameRect.left;
	}
	
		// clip off the right
	if (dstBlitRect.right > dstFrameP->frameRect.right)
	{
		srcBlitRect.right -= dstBlitRect.right - dstFrameP->frameRect.right;
		dstBlitRect.right = dstFrameP->frameRect.right;
	}
	
	
	if (dstBlitRect.bottom <= dstBlitRect.top)	// Make sure height is valid
		return;
		
	if (dstBlitRect.right <= dstBlitRect.left)	// Make sure width is valid
		return;

	
	if (gSWmmuMode != true32b)
	{
		mmuMode = true32b;
		SwapMMUMode(&mmuMode);
	}
		
	// If we didn't have to clip the sprite, we can call the compiled blitter.
	// Hopefully this will be the most common case (i.e. the sprites will be entirely
	// on the screen more often than not)

	if (srcBlitRect.top == srcFrameP->frameRect.top &&
		srcBlitRect.bottom == srcFrameP->frameRect.bottom &&
		srcBlitRect.left == srcFrameP->frameRect.left &&
		srcBlitRect.right == srcFrameP->frameRect.right)
	{
		(*srcFrameP->frameBlitterP)(
			srcFrameP->frameRowBytes,
			dstFrameP->frameRowBytes,
			srcFrameP->frameBaseAddr +
				srcFrameP->scanLinePtrArray[0] + srcRect->left,	// 0 'cause it always starts at top
			dstFrameP->frameBaseAddr +
				dstFrameP->scanLinePtrArray[dstRect->top] + dstRect->left);
	}
	else
	{
		unsigned long numBytesPerRow;
		unsigned long srcBaseOffset;
		
	
			// calculate the offset to the first byte of the source
		srcBaseOffset = srcFrameP->scanLinePtrArray[srcBlitRect.top-srcFrameP->frameRect.top] + 
			srcBlitRect.left;

			// calculate the number of bytes in a row
		numBytesPerRow = (dstBlitRect.right - dstBlitRect.left);

		BlitPixieMask8Bit(
				// calculate the address of the first byte of the source
			(PixelPtr)(srcFrameP->frameBaseAddr + srcBaseOffset),

				// calculate the address of the first byte of the destination
			(PixelPtr)(dstFrameP->frameBaseAddr + 
				(dstFrameP->scanLinePtrArray[dstBlitRect.top]) + dstBlitRect.left),

				// calculate the address of the first byte of the mask
			(PixelPtr)(srcFrameP->maskBaseAddr + srcBaseOffset),

				// calculate the number of rows to blit
			dstBlitRect.bottom - dstBlitRect.top,

				// number of bytes in a row (duh!)
			numBytesPerRow,

				// calculate offset from end of one row to beginning of next row
			srcFrameP->frameRowBytes - numBytesPerRow,
			dstFrameP->frameRowBytes - numBytesPerRow);
	}
	if (gSWmmuMode != true32b)
	{
		SwapMMUMode(&mmuMode);
	}
#endif
}

///--------------------------------------------------------------------------------------
//		BlitPixie8BitGetPixel
///--------------------------------------------------------------------------------------

SW_FUNC unsigned char BlitPixie8BitGetPixel(
	FramePtr srcFrameP,
	Point thePoint )
{
	SInt8 			mmuMode;
	unsigned long 	srcBaseOffset;
	unsigned char	theColor;
	
	if (gSWmmuMode != true32b)
	{
		mmuMode = true32b;
		SwapMMUMode(&mmuMode);
	}

	srcBaseOffset = srcFrameP->scanLinePtrArray[thePoint.v] + 
		thePoint.h+srcFrameP->frameRect.left;
	theColor = *((unsigned char*)(srcFrameP->frameBaseAddr + srcBaseOffset));
		
	if (gSWmmuMode != true32b)
	{
		SwapMMUMode(&mmuMode);
	}
	return theColor;
}

///--------------------------------------------------------------------------------------
//		BlitPixie8BitSetPixel
///--------------------------------------------------------------------------------------

SW_FUNC void BlitPixie8BitSetPixel(
	FramePtr srcFrameP,
	Point thePoint,
	unsigned char theColor)
{
	SInt8 			mmuMode;
	unsigned long 	srcBaseOffset;
	
	if ((thePoint.h >= 0) &&
		(thePoint.h < srcFrameP->frameRect.right-srcFrameP->frameRect.left) &&
		(thePoint.v >= 0) &&
		(thePoint.v < srcFrameP->frameRect.bottom-srcFrameP->frameRect.top) )
	{
	
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}
		srcBaseOffset = srcFrameP->scanLinePtrArray[thePoint.v] + 
			thePoint.h + srcFrameP->frameRect.left;
		*((unsigned char*)(srcFrameP->frameBaseAddr + srcBaseOffset)) = theColor;
			
		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}
	}
}


///--------------------------------------------------------------------------------------
//	SWAnimate8BitStarField
///--------------------------------------------------------------------------------------

SW_FUNC void SWAnimate8BitStarField(
	SpriteWorldPtr	spriteWorldP,
	StarArray		*starArray,
	short			numStars,
	short			backColor)
{
	short			curStar;
	short			horizOffset, vertOffset;	// Offset from window to work area
	SInt8 			mmuMode;
	FramePtr		windFrameP = spriteWorldP->windowFrameP;
	FramePtr		workFrameP = spriteWorldP->workFrameP;
	unsigned long 	srcBaseOffset;

	if (gSWmmuMode != true32b)
	{
		mmuMode = true32b;
		SwapMMUMode(&mmuMode);
	}
	
		// Stores offset if spriteWorld is smaller than window
	horizOffset = windFrameP->frameRect.left;
	vertOffset = windFrameP->frameRect.top;
	
	
	for (curStar = 0; curStar < numStars; curStar++)
	{
			// Make sure we aren't erasing part of a sprite
		if (starArray[curStar].needsToBeErased)
		{
			srcBaseOffset = windFrameP->scanLinePtrArray[starArray[curStar].oldVertLoc] + 
				starArray[curStar].oldHorizLoc;
			*((unsigned char*)(windFrameP->frameBaseAddr + srcBaseOffset)) = backColor;
		}
		
			// Make sure we aren't going to draw over a sprite
		srcBaseOffset = workFrameP->scanLinePtrArray[starArray[curStar].vertLoc - vertOffset] + 
			starArray[curStar].horizLoc - horizOffset;
		if ( *((unsigned char*)(workFrameP->frameBaseAddr + srcBaseOffset)) == backColor)
		{
				// Draw the new star position
			srcBaseOffset = windFrameP->scanLinePtrArray[starArray[curStar].vertLoc] + 
				starArray[curStar].horizLoc;
			*((unsigned char*)(windFrameP->frameBaseAddr + srcBaseOffset)) = 
				starArray[curStar].color;
			
			starArray[curStar].needsToBeErased = true;
		}
		else
		{
			starArray[curStar].needsToBeErased = false;
		}
	}
	
	
	if (gSWmmuMode != true32b)
	{
		SwapMMUMode(&mmuMode);
	}
}


///--------------------------------------------------------------------------------------
//		SWBlitPixie8BitFlipSprite
///--------------------------------------------------------------------------------------

SW_FUNC void SWBlitPixie8BitFlipSprite(
	SpritePtr srcSpriteP )
{
	SInt8 			mmuMode;
	FramePtr		curFrameP;
	unsigned char	holdPixel;
	short			frameCounter, 
					scanLineCounter,
					halfWidth, 
					leftPixel,
					rightPixel,
					frameLeft;
	GDHandle		currentGDH;
	GWorldPtr		currentGWorld;

	
	
	if ( !srcSpriteP->frameArray[0]->isFrameLocked )
	{
		return;
	}
		
	frameCounter = srcSpriteP->numFrames;
	while (frameCounter--)
	{
		if (gSWmmuMode != true32b)
		{
			mmuMode = true32b;
			SwapMMUMode(&mmuMode);
		}


		curFrameP = srcSpriteP->frameArray[frameCounter];
		frameLeft = curFrameP->frameRect.left;
		halfWidth = (curFrameP->frameRect.right - curFrameP->frameRect.left)/2;
		scanLineCounter = curFrameP->numScanLines;
		while (scanLineCounter--)
		{
			rightPixel = leftPixel = halfWidth;
			while (leftPixel--)
			{
				holdPixel = *((unsigned char*)(curFrameP->frameBaseAddr + 
					curFrameP->scanLinePtrArray[scanLineCounter] + leftPixel+frameLeft));
				if ( holdPixel != *((unsigned char*)(curFrameP->frameBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + rightPixel+frameLeft)) )
				{
					*((unsigned char*)(curFrameP->frameBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + leftPixel+frameLeft)) = 
						*((unsigned char*)(curFrameP->frameBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + rightPixel+frameLeft));
					*((unsigned char*)(curFrameP->frameBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + rightPixel+frameLeft)) = 
						holdPixel;					
				}
				rightPixel++;
			}			
		}
		if ( curFrameP->maskPort != NULL )
		{
			scanLineCounter = curFrameP->numScanLines;
			while (scanLineCounter--)
			{
				rightPixel = leftPixel = halfWidth;
				while (leftPixel--)
				{
					holdPixel = *((unsigned char*)(curFrameP->maskBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + leftPixel+frameLeft));
					if ( holdPixel != *((unsigned char*)(curFrameP->maskBaseAddr + 
						curFrameP->scanLinePtrArray[scanLineCounter] + rightPixel+frameLeft)) )
					{
						*((unsigned char*)(curFrameP->maskBaseAddr + 
							curFrameP->scanLinePtrArray[scanLineCounter] + 
							leftPixel+frameLeft)) = 
							*((unsigned char*)(curFrameP->maskBaseAddr + 
							curFrameP->scanLinePtrArray[scanLineCounter] + 
							rightPixel+frameLeft));
						*((unsigned char*)(curFrameP->maskBaseAddr + 
							curFrameP->scanLinePtrArray[scanLineCounter] + 
							rightPixel+frameLeft)) = holdPixel;
					}
					rightPixel++;
				}
			}
		}
		
		if (gSWmmuMode != true32b)
		{
			SwapMMUMode(&mmuMode);
		}

		if ( curFrameP->maskRgn != NULL && curFrameP->maskPort != NULL )
		{
			GetGWorld( &currentGWorld, &currentGDH );
			SetGWorld( curFrameP->maskPort, NULL );
			InvertRect( &(curFrameP->frameRect) );
			(void)SWCreateRegionFromGWorldAndRect( &(curFrameP->maskRgn), 
				curFrameP->maskPort, &(curFrameP->frameRect) );
			SWSetFrameMaskRgn(curFrameP, curFrameP->maskRgn);
			InvertRect( &(curFrameP->frameRect) );
			SetGWorld( currentGWorld, currentGDH );
		}
	}
}


#if SW_USE_C
///--------------------------------------------------------------------------------------
//		BlitPixie8Bit
///--------------------------------------------------------------------------------------
void BlitPixie8Bit(
	PixelChunkPtr srcPixelP,
	PixelChunkPtr dstPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcOffset,
	register unsigned long dstOffset)
{
	register long index;
	register PixelChunkPtr startSrcPixelP;
	register PixelChunkPtr startDstPixelP;
	
		
	startSrcPixelP = srcPixelP;
	startDstPixelP = dstPixelP;

	while (rowsToCopy--)
	{
		register fourblits = (numBytesPerRow >> 2);
		
		srcPixelP = startSrcPixelP;
		dstPixelP = startDstPixelP;
		
		for (index = 0; index < fourblits; index++)
		{
			register unsigned long temp1;
			
			temp1 = srcPixelP[index];
			dstPixelP[index] = temp1;
		}
		srcPixelP += fourblits;
		dstPixelP += fourblits;
		if (numBytesPerRow & 0x2)
			*((unsigned short *) dstPixelP)++ = *((unsigned short *) srcPixelP)++;
		if (numBytesPerRow & 0x1)
			*((unsigned char *)dstPixelP)++ = *((unsigned char *)srcPixelP)++;

			// bump to next row	
		(char *)startSrcPixelP += srcOffset;
		(char *)startDstPixelP += dstOffset;
	}
}


///--------------------------------------------------------------------------------------
//		BlitPixieMask8Bit
///--------------------------------------------------------------------------------------

void BlitPixieMask8Bit(
	register PixelPtr srcPixelP,
	register PixelPtr dstPixelP,
	register PixelPtr maskPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcOffset,
	register unsigned long dstOffset)
{
	register long index;
	register PixelPtr startSrcPixelP;
	register PixelPtr startDstPixelP;
	register PixelPtr startMaskPixelP;
	
	startSrcPixelP = srcPixelP;
	startDstPixelP = dstPixelP;
	startMaskPixelP = maskPixelP;
	
	while (rowsToCopy--)	
	{
		register fourblits = (numBytesPerRow >> 2);
		
		srcPixelP = startSrcPixelP;
		dstPixelP = startDstPixelP;
		maskPixelP = startMaskPixelP;
		
		for (index = 0; index < fourblits; index++)
		{
			register unsigned long temp1;
			
			temp1 = dstPixelP[index] & maskPixelP[index] | srcPixelP[index];
			dstPixelP[index] = temp1;
		}
		srcPixelP += fourblits;
		dstPixelP += fourblits;
		maskPixelP += fourblits;
		if (numBytesPerRow & 0x2)
		{
			register unsigned short temp1;
			
			temp1 = (*((unsigned short *) dstPixelP)) & (*((unsigned short *) maskPixelP)++) |
			*((unsigned short *) srcPixelP)++;
			
			(*((unsigned short *) dstPixelP)++) = temp1;
		}
		if (numBytesPerRow & 0x1)
		{
			register unsigned char temp1;
			
			temp1 = (*((unsigned char *) dstPixelP)) & (*((unsigned char *) maskPixelP)++) |
			*((unsigned char *) srcPixelP)++;
			
			(*((unsigned char *) dstPixelP)++) = temp1;
		}

			// bump to next row
		startSrcPixelP = (PixelPtr)(((char*)startSrcPixelP) + srcOffset);
		startDstPixelP = (PixelPtr)(((char*)startDstPixelP) + dstOffset);
		startMaskPixelP = (PixelPtr)(((char*)startMaskPixelP) + srcOffset);
	}
}


///--------------------------------------------------------------------------------------
//		BlitPixiePartialMask8Bit
///--------------------------------------------------------------------------------------

void BlitPixiePartialMask8Bit(
	register PixelPtr srcPixelP,
	register PixelPtr dstPixelP,
	register PixelPtr maskPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcOffset,
	register unsigned long dstOffset)
{
	register long index;
	register PixelPtr startSrcPixelP;
	register PixelPtr startDstPixelP;
	register PixelPtr startMaskPixelP;
	
	startSrcPixelP = srcPixelP;
	startDstPixelP = dstPixelP;
	startMaskPixelP = maskPixelP;
	
	while (rowsToCopy--)	
	{
		register fourblits = (numBytesPerRow >> 2);
		
		srcPixelP = startSrcPixelP;
		dstPixelP = startDstPixelP;
		maskPixelP = startMaskPixelP;
		
		for (index = 0; index < fourblits; index++)
		{
			register unsigned long temp1;
			
			temp1 = dstPixelP[index] & maskPixelP[index] | 
				( (~maskPixelP[index])&srcPixelP[index]);
			dstPixelP[index] = temp1;
		}
		srcPixelP += fourblits;
		dstPixelP += fourblits;
		maskPixelP += fourblits;
		if (numBytesPerRow & 0x2)
		{
			register unsigned short temp1;
			
			temp1 = (*((unsigned short *) dstPixelP)) & (*((unsigned short *) maskPixelP)) |
			(~(*((unsigned short *) maskPixelP)++) &
			*((unsigned short *) srcPixelP)++);
				
			(*((unsigned short *) dstPixelP)++) = temp1;
		}
		if (numBytesPerRow & 0x1)
		{
			register unsigned char temp1;
			
			temp1 = (*((unsigned char *) dstPixelP)) & (*((unsigned char *) maskPixelP)) |
			(~(*((unsigned char *) maskPixelP)++) &
			*((unsigned char *) srcPixelP)++);
				
			(*((unsigned char *) dstPixelP)++) = temp1;
		}
		
			// bump to next row
		startSrcPixelP = (PixelPtr)(((char*)startSrcPixelP) + srcOffset);
		startDstPixelP = (PixelPtr)(((char*)startDstPixelP) + dstOffset);
		startMaskPixelP = (PixelPtr)(((char*)startMaskPixelP) + srcOffset);
	}
}


#else /* SW_USE_C */

///--------------------------------------------------------------------------------------
//		BlitPixie8Bit
///--------------------------------------------------------------------------------------

SW_ASM_FUNC void BlitPixie8Bit(
	register PixelPtr srcPixelP,
	register PixelPtr dstPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcRowStride,
	register unsigned long dstRowStride)
{
	register unsigned long loopsPerRow;

		SW_ASM_BEGIN

#if __MWERKS__
		fralloc +
#endif

			// longWordsPerRow = numBytesPerRow >> 2;
		move.l	numBytesPerRow, d0
		lsr.l	#2, d0

			// numBytesPerRow -= longWordsPerRow << 2;
		move.l	d0, d1
		lsl.l	#2, d1
		sub.l	d1, numBytesPerRow

			// loopsPerRow = longWordsPerRow >> 4;
		move.l	d0, loopsPerRow
		lsr.l	#4, loopsPerRow

		moveq	#0xF, d1
		and.l	d1, d0
		add.l	d0, d0			// multiply longWordsPerRow by 2 (size of move.l (srcPixelP)+,(dstPixelP)+)
		lea 	@loopEnd, a0	// get address of the end of the loop
		suba.l	d0, a0			// calculate where to jmp in the loop

	@forEachRow:
		move.l	loopsPerRow, d0
		jmp		(a0)
	@loopBase:
		move.l	(srcPixelP)+, (dstPixelP)+	// 16
		move.l	(srcPixelP)+, (dstPixelP)+	// 15
		move.l	(srcPixelP)+, (dstPixelP)+	// 14
		move.l	(srcPixelP)+, (dstPixelP)+	// 13
		move.l	(srcPixelP)+, (dstPixelP)+	// 12
		move.l	(srcPixelP)+, (dstPixelP)+	// 11 
		move.l	(srcPixelP)+, (dstPixelP)+	// 10
		move.l	(srcPixelP)+, (dstPixelP)+	//  9
		move.l	(srcPixelP)+, (dstPixelP)+	//  8
		move.l	(srcPixelP)+, (dstPixelP)+	//  7
		move.l	(srcPixelP)+, (dstPixelP)+	//  6
		move.l	(srcPixelP)+, (dstPixelP)+	//  5
		move.l	(srcPixelP)+, (dstPixelP)+	//  4
		move.l	(srcPixelP)+, (dstPixelP)+	//  3
		move.l	(srcPixelP)+, (dstPixelP)+	//  2
		move.l	(srcPixelP)+, (dstPixelP)+	//  1
	@loopEnd:
		subq.l	#1,d0
		bpl		@loopBase

		// now do any leftover bits
		move.l	numBytesPerRow, d0
		beq		@nextRow
		subq.l	#2, d0
		bmi		@moveByte
		move.w	(srcPixelP)+, (dstPixelP)+
		tst		d0
		beq		@nextRow
	@moveByte:	
		move.b	(srcPixelP)+, (dstPixelP)+

	@nextRow:
		adda.l	srcRowStride, srcPixelP
		adda.l	dstRowStride, dstPixelP
		subq.l	#1, rowsToCopy
		bne		@forEachRow

#if __MWERKS__
		frfree
#endif

		SW_ASM_END
}



///--------------------------------------------------------------------------------------
//		BlitPixieMask8Bit
///--------------------------------------------------------------------------------------

SW_ASM_FUNC void BlitPixieMask8Bit(
	register PixelPtr srcPixelP,
	register PixelPtr dstPixelP,
	register PixelPtr maskPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcRowStride,
	register unsigned long dstRowStride)
{
	register unsigned long loopsPerRow;

		SW_ASM_BEGIN

#if __MWERKS__
		fralloc +
#endif

			// longWordsPerRow = numBytesPerRow >> 2;
		move.l	numBytesPerRow, d0
		lsr.l	#2, d0

			// numBytesPerRow -= longWordsPerRow << 2;
		move.l	d0, d1
		lsl.l	#2, d1
		sub.l	d1, numBytesPerRow

			// loopsPerRow = longWordsPerRow >> 4;
		move.l	d0, loopsPerRow
		lsr.l	#4, loopsPerRow

			
		moveq	#0xF, d1
		and.l	d1, d0
		lsl.l	#3, d0					// longWordsPerRow *= 8;
		lea 	@loopEnd, a0			// get address of the end of the loop
		sub.l	d0, a0					// calculate where to jmp in the loop

	@forEachRow:
		move.l	loopsPerRow, d2
		jmp		(a0)
	@loopBase:
			// 16
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
			// 15
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
			// 14
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	// 13
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	// 12
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	// 11
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	// 10
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  9
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  8
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  7
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
			//  6
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  5
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  4
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  3
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
		//  2
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
      	//  1
		move.l	(dstPixelP), d0
		and.l	(maskPixelP)+, d0
		or.l	(srcPixelP)+, d0
		move.l	d0, (dstPixelP)+
	@loopEnd:
		subq.l	#1, d2
		bpl		@loopBase

		// now do any leftover bits
		move.l	numBytesPerRow, d2
		beq		@nextRow
		subq.l	#2, d2
		bmi		@moveByte
		move.w	(dstPixelP), d0
		and.w	(maskPixelP)+, d0
		or.w	(srcPixelP)+, d0
		move.w	d0, (dstPixelP)+
		tst		d2
		beq		@nextRow
	@moveByte:	
		move.b	(dstPixelP), d0
		and.b	(maskPixelP)+, d0
		or.b	(srcPixelP)+, d0
		move.b	d0, (dstPixelP)+

	@nextRow:
		adda.l	srcRowStride, srcPixelP
		adda.l	srcRowStride, maskPixelP
		adda.l	dstRowStride, dstPixelP
		subq.l	#1, rowsToCopy
		bne		@forEachRow

#if __MWERKS__
		frfree
#endif

		SW_ASM_END
}




///--------------------------------------------------------------------------------------
//		BlitPixiePartialMask8Bit
///--------------------------------------------------------------------------------------

SW_ASM_FUNC void BlitPixiePartialMask8Bit(
	register PixelPtr srcPixelP,
	register PixelPtr dstPixelP,
	register PixelPtr maskPixelP,
	register unsigned long rowsToCopy,
	register unsigned long numBytesPerRow,
	register unsigned long srcRowStride,
	register unsigned long dstRowStride)
{
	register unsigned long loopsPerRow;

		SW_ASM_BEGIN

#if __MWERKS__
		fralloc +
#endif

			// longWordsPerRow = numBytesPerRow >> 2;
		move.l	numBytesPerRow, d0
		lsr.l	#2, d0

			// numBytesPerRow -= longWordsPerRow << 2;
		move.l	d0, d1
		lsl.l	#2, d1
		sub.l	d1, numBytesPerRow

			// loopsPerRow = longWordsPerRow >> 4;
		move.l	d0, loopsPerRow
		lsr.l	#4, loopsPerRow

			
		moveq	#0xF, d1
		and.l	d1, d0
		mulu	#14, d0					// longWordsPerRow *= 14 (bytes in segment of loop)
		lea 	@loopEnd, a0			// get address of the end of the loop
		sub.l	d0, a0					// calculate where to jmp in the loop

	@forEachRow:
		move.l	loopsPerRow, d2
		jmp		(a0)
	@loopBase:
			// 16
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
			// 15
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
			// 14
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	// 13
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	// 12
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	// 11
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	// 10
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  9
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  8
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  7
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
			//  6
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  5
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  4
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  3
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
		//  2
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
      	//  1
		move.l	(dstPixelP), d0
		and.l	(maskPixelP), d0
		move.l	(maskPixelP)+, d1
		not.l	d1
		and.l	(srcPixelP)+, d1
		or.l	d1, d0
		move.l	d0, (dstPixelP)+
	@loopEnd:
		subq.l	#1, d2
		bpl		@loopBase

		// now do any leftover bits
		move.l	numBytesPerRow, d2
		beq		@nextRow
		subq.l	#2, d2
		bmi		@moveByte
		move.w	(dstPixelP), d0
		and.w	(maskPixelP), d0
		move.w	(maskPixelP)+, d1
		not.w	d1
		and.w	(srcPixelP)+, d1
		or.w	d1, d0
		move.w	d0, (dstPixelP)+
		tst		d2
		beq		@nextRow
	@moveByte:
		move.b	(dstPixelP), d0
		and.b	(maskPixelP), d0
		move.b	(maskPixelP)+, d1
		not.b	d1
		and.b	(srcPixelP)+, d1
		or.b	d1, d0
		move.b	d0, (dstPixelP)+

	@nextRow:
		adda.l	srcRowStride, srcPixelP
		adda.l	srcRowStride, maskPixelP
		adda.l	dstRowStride, dstPixelP
		subq.l	#1, rowsToCopy
		bne		@forEachRow

#if __MWERKS__
		frfree
#endif

		SW_ASM_END
}


#endif /* SW_USE_C */


