///--------------------------------------------------------------------------------------
//	SpriteWorld.c
//
//	Portions are copyright:  1991-94 Tony Myles, All rights reserved worldwide.
//
//	Description:	implementation of the sprite world architecture
///--------------------------------------------------------------------------------------


#ifndef __SWCOMMON__
#include "SWCommonHeaders.h"
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __GESTALTEQU__
#include <GestaltEqu.h>
#endif

#ifndef __TIMER__
#include <Timer.h>
#endif

#ifndef __SPRITEWORLDUTILS__
#include "SpriteWorldUtils.h"
#endif

#ifndef __SPRITEWORLD__
#include "SpriteWorld.h"
#endif

#ifndef __SPRITELAYER__
#include "SpriteLayer.h"
#endif

#ifndef __SPRITE__
#include "Sprite.h"
#endif

#ifndef __SPRITEFRAME__
#include "SpriteFrame.h"
#endif

#ifndef __BLITPIXIE__
#include "BlitPixie.h"
#endif

#ifndef __RETRACE__
#include <Retrace.h>
#endif

#ifndef __SCROLLING__
#include "Scrolling.h"
#endif

#ifndef __TILING__
#include "Tiling.h"
#endif


char 			gSWmmuMode;
RgnHandle 		gCollisionSectRgn = NULL;
RgnHandle 		gCollisionSpareRgn = NULL;


///--------------------------------------------------------------------------------------
//	SWEnterSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWEnterSpriteWorld(void)
{
	OSErr	err = noErr;

	gSWmmuMode = GetMMUMode();
	
	if ( !SWHasSystem7() )
	{
		err = kSystemTooOldErr;
	}	
	
	gCollisionSectRgn = NewRgn();
	gCollisionSpareRgn = NewRgn();
	
	SWSetStickyIfError( err );
	return err;
}



///--------------------------------------------------------------------------------------
//	SWCreateSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateSpriteWorld(
	GDHandle			mainGDH,
	SpriteWorldPtr		*spriteWorldP,
	FramePtr			windowFrameP,
	FramePtr			backFrameP,
	FramePtr			workFrameP)
{
	OSErr			err;
	SpriteWorldPtr	tempSpriteWorldP;
	PixMapHandle	basePixMap;
	Rect			*visScrollRectP;
	

	err = noErr;
	*spriteWorldP = NULL;

	tempSpriteWorldP = (SpriteWorldPtr)NewPtrClear((Size)sizeof(SpriteWorldRec));

	if (tempSpriteWorldP != NULL)
	{
		basePixMap = (**mainGDH).gdPMap;
		tempSpriteWorldP->pixelDepth = (**basePixMap).pixelSize;
		tempSpriteWorldP->mainSWGDH = mainGDH;
		tempSpriteWorldP->windowFrameP = windowFrameP;
		tempSpriteWorldP->backFrameP = backFrameP;
		tempSpriteWorldP->workFrameP = workFrameP;
		tempSpriteWorldP->offscreenDrawProc = SWStdWorldDrawProc;
		tempSpriteWorldP->screenDrawProc = SWStdWorldDrawProc;
		tempSpriteWorldP->windRect = windowFrameP->frameRect;
		tempSpriteWorldP->backRect = backFrameP->frameRect;
		
		tempSpriteWorldP->scrollRectMoveBounds.left = 0;
		tempSpriteWorldP->scrollRectMoveBounds.top = 0;
		tempSpriteWorldP->scrollRectMoveBounds.right = 32767;
		tempSpriteWorldP->scrollRectMoveBounds.bottom = 32767;
		tempSpriteWorldP->horizScrollRectOffset = 0;
		tempSpriteWorldP->vertScrollRectOffset = 0;
		tempSpriteWorldP->horizScrollDelta = 0;
		tempSpriteWorldP->vertScrollDelta = 0;
		tempSpriteWorldP->worldMoveProc = NULL;
		
			// Make sure visScrollRect is at 0,0 top-left corner
		tempSpriteWorldP->visScrollRect = windowFrameP->frameRect;
		visScrollRectP = &tempSpriteWorldP->visScrollRect;
		OffsetRect(visScrollRectP, -visScrollRectP->left, -visScrollRectP->top);
		tempSpriteWorldP->offscreenScrollRect = tempSpriteWorldP->visScrollRect;
		tempSpriteWorldP->oldVisScrollRect = tempSpriteWorldP->visScrollRect;
		
		tempSpriteWorldP->tileMap = NULL;
		tempSpriteWorldP->tilingIsInitialized = false;
		tempSpriteWorldP->tilingIsOn = false;
		tempSpriteWorldP->numTilesChanged = 0;
		tempSpriteWorldP->tileChangeProc = NULL;
		tempSpriteWorldP->tileMaskDrawProc = SWStdSpriteDrawProc;
		
		tempSpriteWorldP->frameHasOccured = false;
		tempSpriteWorldP->fpsTimeInterval = 0L;
		
		tempSpriteWorldP->lastMicroseconds.hi = 0L;
		tempSpriteWorldP->lastMicroseconds.lo = 0L;
		tempSpriteWorldP->runningTimeCount = 0L;
		
		tempSpriteWorldP->vblTaskRec.hasVBLFired = true;
		tempSpriteWorldP->usingVBL = false;
		
		SWClearStickyError();

		*spriteWorldP = tempSpriteWorldP;
	}
	else
	{
		err = MemError();
	}

	return err;
}


///--------------------------------------------------------------------------------------
//	SWCreateSpriteWorldFromWindow
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateSpriteWorldFromWindow(
	SpriteWorldPtr*	spriteWorldP,
	CWindowPtr		srcWindowP,
	Rect*			worldRectP,
	Rect*			offscreenRectP)
{
	OSErr 		err = noErr;
	FramePtr 	windowFrameP, backFrameP, workFrameP;
	Rect 		tempRect, globalRect;
	GDHandle	mainGDH;
	
	
	*spriteWorldP = NULL;
	windowFrameP = backFrameP = workFrameP = NULL;
	
	SetPort( (WindowPtr)srcWindowP );
	
	tempRect = (worldRectP == NULL) ? srcWindowP->portRect : *worldRectP;
	
	globalRect = tempRect;
	LocalToGlobal((Point *)&globalRect.top);
	LocalToGlobal((Point *)&globalRect.bottom);
	
	mainGDH = GetMaxDevice( &globalRect );
	
	if ( srcWindowP->portVersion < (short)0xC000 )
	{
		err = kNotCWindowErr;
	}
	
		// create window frame
	if (err == noErr)
	{
		err = SWCreateWindowFrame(&windowFrameP, &tempRect);
	}
	
		// If a custom offscreenRect has been requested, use it instead.
	if (offscreenRectP != NULL)
		tempRect = *offscreenRectP;
	
		// offset Rect to (0,0) for other 2 frames
	OffsetRect(&tempRect, -tempRect.left, -tempRect.top);
	
	if (err == noErr)
	{
			// create back drop frame
		err = SWCreateFrame(mainGDH, &backFrameP, &tempRect);
	}

	if (err == noErr)
	{
			// create loader frame
		err = SWCreateFrame(mainGDH, &workFrameP, &tempRect);
	}


	if (err == noErr)
	{
			// create sprite world
		err = SWCreateSpriteWorld(mainGDH, spriteWorldP, windowFrameP, backFrameP, workFrameP);
	}

	if (err != noErr)
	{
			// an error occurred so dispose of anything we managed to create

		if (windowFrameP != NULL)
		{
			SWDisposeFrame(windowFrameP);
		}

		if (backFrameP != NULL)
		{
			SWDisposeFrame(backFrameP);
		}

		if (workFrameP != NULL)
		{
			SWDisposeFrame(workFrameP);
		}
	}
	
	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
//	SWDisposeSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWDisposeSpriteWorld(
	SpriteWorldPtr	spriteWorldP)
{
	SpriteLayerPtr 	curLayerP;
	SpritePtr 		curSpriteP;
	
	
	if (spriteWorldP != NULL)
	{
		SWUnlockSpriteWorld(spriteWorldP);
			
		while ((curLayerP = SWGetNextSpriteLayer(spriteWorldP, NULL)) != NULL)
		{		
				// dispose of each sprite in the layer
			while ((curSpriteP = SWGetNextSprite(curLayerP, NULL)) != NULL)
			{
				SWRemoveSprite(curLayerP, curSpriteP);
				SWDisposeSprite(curSpriteP);
			}
			
				// dispose of each layer in the spriteWorld
			SWRemoveSpriteLayer(spriteWorldP, curLayerP);
			SWDisposeSpriteLayer(curLayerP);
		}

		SWDisposeFrame(spriteWorldP->backFrameP);
		SWDisposeFrame(spriteWorldP->workFrameP);

		spriteWorldP->windowFrameP->framePort = NULL;
		SWDisposeFrame(spriteWorldP->windowFrameP);
		
		SWExitTiling(spriteWorldP);
		SWSyncSpriteWorldToVBL(spriteWorldP, false);
		
		DisposePtr((Ptr)spriteWorldP);
	}
}


///--------------------------------------------------------------------------------------
//	SWExitSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWExitSpriteWorld(void)
{
	if (gCollisionSectRgn != NULL)
		DisposeRgn(gCollisionSectRgn);
	if (gCollisionSpareRgn != NULL)
		DisposeRgn(gCollisionSpareRgn);
}


///--------------------------------------------------------------------------------------
//	SWAddSpriteLayer
///--------------------------------------------------------------------------------------

SW_FUNC void SWAddSpriteLayer(
	SpriteWorldPtr		spriteWorldP,
	SpriteLayerPtr		newSpriteLayerP)
{
	SpriteLayerPtr 	tailSpriteLayerP = spriteWorldP->tailSpriteLayerP;

	if (tailSpriteLayerP != NULL)
	{
			// doubly link the new layer
		tailSpriteLayerP->nextSpriteLayerP = newSpriteLayerP;
		newSpriteLayerP->prevSpriteLayerP = tailSpriteLayerP;
		newSpriteLayerP->nextSpriteLayerP = NULL;
	}	
	else
	{
		newSpriteLayerP->prevSpriteLayerP = NULL;
		newSpriteLayerP->nextSpriteLayerP = NULL;

			// make the new layer the head
		spriteWorldP->headSpriteLayerP = newSpriteLayerP;
	}

		// make the new layer the tail
	spriteWorldP->tailSpriteLayerP = newSpriteLayerP;
}


///--------------------------------------------------------------------------------------
//	SWRemoveSpriteLayer
///--------------------------------------------------------------------------------------

SW_FUNC void SWRemoveSpriteLayer(
	SpriteWorldPtr	spriteWorldP,
	SpriteLayerPtr	oldSpriteLayerP)
{	
		// is there a next layer?
	if (oldSpriteLayerP->nextSpriteLayerP != NULL)
	{
			// link the next layer to the prev layer
		oldSpriteLayerP->nextSpriteLayerP->prevSpriteLayerP = oldSpriteLayerP->prevSpriteLayerP;
	}
	else
	{
			// make the prev layer the tail
		spriteWorldP->tailSpriteLayerP = oldSpriteLayerP->prevSpriteLayerP;
	}

		// is there a prev layer?
	if (oldSpriteLayerP->prevSpriteLayerP != NULL)
	{
			// link the prev layer to the next layer
		oldSpriteLayerP->prevSpriteLayerP->nextSpriteLayerP = oldSpriteLayerP->nextSpriteLayerP;
	}
	else
	{
			// make the next layer the head
		spriteWorldP->headSpriteLayerP = oldSpriteLayerP->nextSpriteLayerP;
	}
}


///--------------------------------------------------------------------------------------
//	SWSwapSpriteLayer
///--------------------------------------------------------------------------------------

SW_FUNC void SWSwapSpriteLayer(
	SpriteWorldPtr	spriteWorldP,
	SpriteLayerPtr	srcSpriteLayerP,
	SpriteLayerPtr	dstSpriteLayerP)
{
	register SpriteLayerPtr		swapSpriteLayerP;
	
	
		// adjacent Layers are a special case
		
	if ( srcSpriteLayerP->nextSpriteLayerP == dstSpriteLayerP ||
		dstSpriteLayerP->nextSpriteLayerP == srcSpriteLayerP )
	{
		if ( srcSpriteLayerP->nextSpriteLayerP == dstSpriteLayerP )
		{
			if ( srcSpriteLayerP->prevSpriteLayerP != NULL )
				(srcSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP = dstSpriteLayerP;
			if ( dstSpriteLayerP->nextSpriteLayerP != NULL )
				(dstSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP = srcSpriteLayerP;
			
			dstSpriteLayerP->prevSpriteLayerP = srcSpriteLayerP->prevSpriteLayerP;
			srcSpriteLayerP->nextSpriteLayerP = dstSpriteLayerP->nextSpriteLayerP;
			
			dstSpriteLayerP->nextSpriteLayerP = srcSpriteLayerP;
			srcSpriteLayerP->prevSpriteLayerP = dstSpriteLayerP;
		}
		else
		{
			if ( dstSpriteLayerP->prevSpriteLayerP != NULL )
				(dstSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP = srcSpriteLayerP;
			if ( srcSpriteLayerP->nextSpriteLayerP != NULL )
				(srcSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP = dstSpriteLayerP;
			
			srcSpriteLayerP->prevSpriteLayerP = dstSpriteLayerP->prevSpriteLayerP;
			dstSpriteLayerP->nextSpriteLayerP = srcSpriteLayerP->nextSpriteLayerP;
			
			srcSpriteLayerP->nextSpriteLayerP = dstSpriteLayerP;
			dstSpriteLayerP->prevSpriteLayerP = srcSpriteLayerP;
		}
	}
	else
	{
		if ( srcSpriteLayerP->prevSpriteLayerP != NULL && 
				dstSpriteLayerP->prevSpriteLayerP != NULL )
		{
			swapSpriteLayerP = (srcSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP;
			(srcSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP = 
					(dstSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP;
			(dstSpriteLayerP->prevSpriteLayerP)->nextSpriteLayerP = swapSpriteLayerP;
		}
		
		if ( srcSpriteLayerP->nextSpriteLayerP != NULL && 
				dstSpriteLayerP->nextSpriteLayerP != NULL )
		{
			swapSpriteLayerP = (srcSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP;
			(srcSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP = 
					(dstSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP;
			(dstSpriteLayerP->nextSpriteLayerP)->prevSpriteLayerP = swapSpriteLayerP;
		}
		swapSpriteLayerP = srcSpriteLayerP->nextSpriteLayerP;
		srcSpriteLayerP->nextSpriteLayerP = dstSpriteLayerP->nextSpriteLayerP;
		dstSpriteLayerP->nextSpriteLayerP = swapSpriteLayerP;

		swapSpriteLayerP = srcSpriteLayerP->prevSpriteLayerP;
		srcSpriteLayerP->prevSpriteLayerP = dstSpriteLayerP->prevSpriteLayerP;
		dstSpriteLayerP->prevSpriteLayerP = swapSpriteLayerP;
	}

	if (srcSpriteLayerP->nextSpriteLayerP == NULL)
	{
		spriteWorldP->tailSpriteLayerP = srcSpriteLayerP;
	}
	else if (srcSpriteLayerP->prevSpriteLayerP == NULL)
	{
		spriteWorldP->headSpriteLayerP = srcSpriteLayerP;
	}

	if (dstSpriteLayerP->nextSpriteLayerP == NULL)
	{
		spriteWorldP->tailSpriteLayerP = dstSpriteLayerP;
	}
	else if (dstSpriteLayerP->prevSpriteLayerP == NULL)
	{
		spriteWorldP->headSpriteLayerP = dstSpriteLayerP;
	}
}


///--------------------------------------------------------------------------------------
//	SWGetNextSpriteLayer
///--------------------------------------------------------------------------------------

SW_FUNC SpriteLayerPtr SWGetNextSpriteLayer(
	SpriteWorldPtr	spriteWorldP,
	SpriteLayerPtr	curSpriteLayerP)
{
	return (curSpriteLayerP == NULL) ?
			spriteWorldP->headSpriteLayerP :
			curSpriteLayerP->nextSpriteLayerP;
}


///--------------------------------------------------------------------------------------
//	SWLockSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWLockSpriteWorld(
	SpriteWorldPtr	spriteWorldP)
{
	SpriteLayerPtr curSpriteLayerP;

	SWLockFrame(spriteWorldP->windowFrameP);
	SWLockFrame(spriteWorldP->workFrameP);
	SWLockFrame(spriteWorldP->backFrameP);
	
	SWLockTiles(spriteWorldP);

	curSpriteLayerP = spriteWorldP->headSpriteLayerP;

	while (curSpriteLayerP != NULL)
	{
		SWLockSpriteLayer(curSpriteLayerP);

		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
}


///--------------------------------------------------------------------------------------
//	SWUnlockSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWUnlockSpriteWorld(
	SpriteWorldPtr	spriteWorldP)
{
	SpriteLayerPtr curSpriteLayerP;

	SWUnlockFrame(spriteWorldP->windowFrameP);
	SWUnlockFrame(spriteWorldP->backFrameP);
	SWUnlockFrame(spriteWorldP->workFrameP);
	
	SWUnlockTiles(spriteWorldP);

	curSpriteLayerP = spriteWorldP->headSpriteLayerP;

	while (curSpriteLayerP != NULL)
	{
		SWUnlockSpriteLayer(curSpriteLayerP);

		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
}


///--------------------------------------------------------------------------------------
//	SWSetPortToBackground
///--------------------------------------------------------------------------------------

SW_FUNC void SWSetPortToBackground(
	SpriteWorldPtr	spriteWorldP)
{
	if ( spriteWorldP->backFrameP->isFrameLocked )
		SetGWorld(spriteWorldP->backFrameP->framePort, nil);
}


///--------------------------------------------------------------------------------------
//	SWSetPortToWorkArea
///--------------------------------------------------------------------------------------

SW_FUNC void SWSetPortToWorkArea(
	SpriteWorldPtr	spriteWorldP)
{
	if ( spriteWorldP->workFrameP->isFrameLocked )
		SetGWorld(spriteWorldP->workFrameP->framePort, nil);
}


///--------------------------------------------------------------------------------------
//	SWSetPortToWindow
///--------------------------------------------------------------------------------------

SW_FUNC void SWSetPortToWindow(
	SpriteWorldPtr	spriteWorldP)
{
	SetGWorld(spriteWorldP->windowFrameP->framePort, nil);
}


///--------------------------------------------------------------------------------------
//	SWSetSpriteWorldOffscreenDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWSetSpriteWorldOffscreenDrawProc(
	SpriteWorldPtr	spriteWorldP,
	DrawProcPtr		drawProc)
{
	OSErr	err = noErr;
	
	if (spriteWorldP->pixelDepth != 8)
	{
		if ( drawProc == BlitPixie8BitRectDrawProc || 
			 drawProc == BP8BitInterlacedRectDrawProc )
			err = kWrongDepthErr;
	}

	if ( err == noErr )
		spriteWorldP->offscreenDrawProc = drawProc;
	
	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
//	SWSetSpriteWorldScreenDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWSetSpriteWorldScreenDrawProc(
	SpriteWorldPtr	spriteWorldP,
	DrawProcPtr		drawProc)
{
	OSErr	err = noErr;
	
	if (spriteWorldP->pixelDepth != 8)
	{
		if ( drawProc == BlitPixie8BitRectDrawProc || 
			 drawProc == BP8BitInterlacedRectDrawProc )
			err = kWrongDepthErr;
	}

	if ( err == noErr )
		spriteWorldP->screenDrawProc = drawProc;
		
	SWSetStickyIfError( err );
	return err;
}

///--------------------------------------------------------------------------------------
//	SWStdWorldDrawProc
///--------------------------------------------------------------------------------------

SW_FUNC void SWStdWorldDrawProc(
	FramePtr	srcFrameP,
	FramePtr	dstFrameP,
	Rect*		srcRect,
	Rect*		dstRect)
{	
	Rect	srcCopyBitsRect = *srcRect;
	Rect	dstCopyBitsRect = *dstRect;
	
	
		// clip off the top
	if (dstCopyBitsRect.top < dstFrameP->frameRect.top)
	{
		srcCopyBitsRect.top += dstFrameP->frameRect.top - dstCopyBitsRect.top;
		dstCopyBitsRect.top =  dstFrameP->frameRect.top;
		if (dstCopyBitsRect.top >= dstCopyBitsRect.bottom) return;
	}
		// clip off the bottom
	if (dstCopyBitsRect.bottom > dstFrameP->frameRect.bottom)
	{
		srcCopyBitsRect.bottom -= dstCopyBitsRect.bottom - dstFrameP->frameRect.bottom;
		dstCopyBitsRect.bottom = dstFrameP->frameRect.bottom;
		if (dstCopyBitsRect.bottom <= dstCopyBitsRect.top) return;
	}
		// clip off the left
	if (dstCopyBitsRect.left < dstFrameP->frameRect.left)
	{
		srcCopyBitsRect.left += dstFrameP->frameRect.left - dstCopyBitsRect.left;
		dstCopyBitsRect.left = dstFrameP->frameRect.left;
		if (dstCopyBitsRect.left >= dstCopyBitsRect.right) 
			return;
	}
		// clip off the right
	if (dstCopyBitsRect.right > dstFrameP->frameRect.right)
	{
		srcCopyBitsRect.right -= dstCopyBitsRect.right - dstFrameP->frameRect.right;
		dstCopyBitsRect.right = dstFrameP->frameRect.right;
		if (dstCopyBitsRect.right <= dstCopyBitsRect.left) return;
	}
			
	CopyBits( (BitMap*)srcFrameP->framePix,
		(BitMap*)dstFrameP->framePix,
		&srcCopyBitsRect,
		&dstCopyBitsRect,
		srcCopy, NULL);
}


///--------------------------------------------------------------------------------------
//	SWCopyBackgroundToWorkArea
///--------------------------------------------------------------------------------------

SW_FUNC void SWCopyBackgroundToWorkArea(
	SpriteWorldPtr spriteWorldP)
{		
	GWorldPtr		holdGWorld;
	GDHandle		holdGDH;
	
	GetGWorld( &holdGWorld, &holdGDH );
	
		// Copy the background frame into the work area
	
	SetGWorld(spriteWorldP->workFrameP->framePort, nil);
	(*spriteWorldP->offscreenDrawProc)(spriteWorldP->backFrameP,
								spriteWorldP->workFrameP,
								&spriteWorldP->backFrameP->frameRect,
								&spriteWorldP->workFrameP->frameRect);
	
	SetGWorld( holdGWorld, holdGDH );
}


///--------------------------------------------------------------------------------------
//	SWUpdateWindow
///--------------------------------------------------------------------------------------

SW_FUNC void SWUpdateWindow(
	SpriteWorldPtr spriteWorldP)
{
	GWorldPtr		holdGWorld;
	GDHandle		holdGDH;
	
	GetGWorld( &holdGWorld, &holdGDH );
	
	SetGWorld(spriteWorldP->windowFrameP->framePort, nil);
	
	
	spriteWorldP->numTilesChanged = 0;
	
	
	if ( spriteWorldP->usingVBL )
	{  
		spriteWorldP->vblTaskRec.hasVBLFired = false;
		while ( !spriteWorldP->vblTaskRec.hasVBLFired )
		{}
	}
	
	(*spriteWorldP->screenDrawProc)(spriteWorldP->workFrameP,
									spriteWorldP->windowFrameP,
									&spriteWorldP->visScrollRect,
									&spriteWorldP->windowFrameP->frameRect);
									
	SetGWorld( holdGWorld, holdGDH );
}


///--------------------------------------------------------------------------------------
//	SWUpdateSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWUpdateSpriteWorld(
	SpriteWorldPtr spriteWorldP,
	Boolean updateWindow)
{
	register 	SpriteLayerPtr curSpriteLayerP;
	register 	SpritePtr curSpriteP;
	Rect		tempRect;
	GWorldPtr	holdGWorld;
	GDHandle	holdGDH;
	
	
	GetGWorld( &holdGWorld, &holdGDH );
	
	
	spriteWorldP->numTilesChanged = 0;

		// Copy the background into the work area	
	SetGWorld(spriteWorldP->workFrameP->framePort, nil);
	(*spriteWorldP->offscreenDrawProc)(spriteWorldP->backFrameP,
									   spriteWorldP->workFrameP,
									   &spriteWorldP->backFrameP->frameRect,
									   &spriteWorldP->workFrameP->frameRect);
	
		// Build the current frame of the animation in the work area

	curSpriteLayerP = spriteWorldP->headSpriteLayerP;

		// iterate through the layers in this world
	while (curSpriteLayerP != NULL)
	{
		curSpriteP = curSpriteLayerP->headSpriteP;

			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{
			if (curSpriteP->isVisible)
			{
					// draw the sprite in the work area
				(*curSpriteP->frameDrawProc)(curSpriteP->curFrameP,
											spriteWorldP->workFrameP,
											&curSpriteP->curFrameP->frameRect,
											&curSpriteP->destFrameRect);
				
				if (curSpriteP->isUnderTiles && spriteWorldP->tilingIsOn)
				{
					tempRect = curSpriteP->destFrameRect;
					
						// Clip tempRect to visScrollRect
					if (tempRect.left < spriteWorldP->visScrollRect.left)
						tempRect.left = spriteWorldP->visScrollRect.left;
					if (tempRect.right > spriteWorldP->visScrollRect.right)
						tempRect.right = spriteWorldP->visScrollRect.right;
					if (tempRect.top < spriteWorldP->visScrollRect.top)
						tempRect.top = spriteWorldP->visScrollRect.top;
					if (tempRect.bottom > spriteWorldP->visScrollRect.bottom)
						tempRect.bottom = spriteWorldP->visScrollRect.bottom;
	
					SWDrawTilesAboveSprite(spriteWorldP, &tempRect);
				}
			
			}

			curSpriteP = curSpriteP->nextSpriteP;
		}

		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
	
		// Copy the work area into the window
	
	if (updateWindow)
	{
		SetGWorld(spriteWorldP->windowFrameP->framePort, nil);
		
		if ( spriteWorldP->usingVBL )
		{  
			spriteWorldP->vblTaskRec.hasVBLFired = false;
			while ( !spriteWorldP->vblTaskRec.hasVBLFired )
			{}
		}
		
		(*spriteWorldP->screenDrawProc)(spriteWorldP->workFrameP,
										spriteWorldP->windowFrameP,
										&spriteWorldP->visScrollRect,
										&spriteWorldP->windowFrameP->frameRect);
	}
	
	SetGWorld( holdGWorld, holdGDH );
}


///--------------------------------------------------------------------------------------
//	SWProcessSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWProcessSpriteWorld(
	SpriteWorldPtr spriteWorldP)
{
	UnsignedWide				curMicroseconds;
	register unsigned long		millisecDifference;
	register SpriteLayerPtr		curSpriteLayerP;
	register SpritePtr			curSpriteP, nextSpriteP;
	register FramePtr			newFrameP;
	register short				horizOffset, vertOffset;
	
	
	Microseconds( &curMicroseconds );
		// has hi long has rolled over?
	if ( curMicroseconds.hi > spriteWorldP->lastMicroseconds.hi )
	{
		millisecDifference = 
			(0xffffffff - (spriteWorldP->lastMicroseconds.lo - curMicroseconds.lo))/1000;
	}
	else
	{
		millisecDifference = (curMicroseconds.lo - spriteWorldP->lastMicroseconds.lo)/1000;
	}
	if ( millisecDifference > 0 )
	{
		spriteWorldP->runningTimeCount += millisecDifference;
		spriteWorldP->lastMicroseconds.lo = curMicroseconds.lo;
		spriteWorldP->lastMicroseconds.hi = curMicroseconds.hi;
	}

	if ( (spriteWorldP->runningTimeCount - spriteWorldP->timeOfLastFrame > 
				spriteWorldP->fpsTimeInterval) )
	{
		spriteWorldP->frameHasOccured = true;
		spriteWorldP->timeOfLastFrame = spriteWorldP->runningTimeCount;
	}
	else
	{
		spriteWorldP->frameHasOccured = false;
		return;
	}

	if ( spriteWorldP->anySpritesNeedRemoving )
	{
		SWFindSpritesToBeRemoved( spriteWorldP );
	}
	
		// Call the tileChangeProc, if there is one
	if (spriteWorldP->tileChangeProc != NULL)
	{
		(*spriteWorldP->tileChangeProc)(spriteWorldP);
	}
	
	curSpriteLayerP = spriteWorldP->headSpriteLayerP;	
	
		// iterate through the layers in this world
	while (curSpriteLayerP != NULL)
	{
		curSpriteP = curSpriteLayerP->headSpriteP;
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{			
			nextSpriteP = curSpriteP->nextSpriteP;
			
				// is it time to advance the sprites frame?
			if ( curSpriteP->frameTimeInterval >= 0 && 
				(spriteWorldP->runningTimeCount - curSpriteP->timeOfLastFrameChange > 
				curSpriteP->frameTimeInterval) )
			{
				register long		currentFrameIndex;

									
				curSpriteP->timeOfLastFrameChange = spriteWorldP->runningTimeCount;
				
				currentFrameIndex = curSpriteP->curFrameIndex;
				currentFrameIndex+= curSpriteP->frameAdvance;
				
				if (currentFrameIndex < curSpriteP->firstFrameIndex)
				{
						// wrap to the last frame
					currentFrameIndex = curSpriteP->lastFrameIndex;
				}
				else if (currentFrameIndex > curSpriteP->lastFrameIndex)
				{
						// wrap to the first frame
					currentFrameIndex = curSpriteP->firstFrameIndex;
				}
				
					// get new frame
				newFrameP =	curSpriteP->frameArray[currentFrameIndex];
				
				curSpriteP->curFrameIndex = currentFrameIndex;
					// is there a frame callback?
				if (curSpriteP->frameChangeProc != NULL)
				{
						// call it
					(*curSpriteP->frameChangeProc)(curSpriteP, newFrameP,
							&curSpriteP->curFrameIndex);
					
						// make sure the new frame index is in range
					if (curSpriteP->curFrameIndex < 0)
					{
						curSpriteP->curFrameIndex = 0;
					}
					else if (curSpriteP->curFrameIndex >= curSpriteP->maxFrames)
					{
						curSpriteP->curFrameIndex = curSpriteP->maxFrames - 1;
					}
				}
				
					// change the frame
				newFrameP =	curSpriteP->frameArray[curSpriteP->curFrameIndex];
				
					// has the frame actually changed?
				if (curSpriteP->curFrameP != newFrameP)
				{
					curSpriteP->curFrameP = newFrameP;
					
					horizOffset = (curSpriteP->destFrameRect.left - newFrameP->frameRect.left);
					vertOffset = (curSpriteP->destFrameRect.top - newFrameP->frameRect.top);
					
					curSpriteP->destFrameRect = newFrameP->frameRect;
					
					curSpriteP->destFrameRect.left += horizOffset;
					curSpriteP->destFrameRect.right += horizOffset;
					curSpriteP->destFrameRect.top += vertOffset;
					curSpriteP->destFrameRect.bottom += vertOffset;

					curSpriteP->needsToBeDrawn = true;
				}
			}
				// is it time to move the sprite?
			if ( curSpriteP->moveTimeInterval >= 0 && 
				(spriteWorldP->runningTimeCount - curSpriteP->timeOfLastMove) > 
				curSpriteP->moveTimeInterval )
			{
				curSpriteP->timeOfLastMove = spriteWorldP->runningTimeCount;
				
					// is there a movement callback?
				if (curSpriteP->spriteMoveProc != NULL)
				{
					(*curSpriteP->spriteMoveProc)(curSpriteP);
				}
			}
			curSpriteP = nextSpriteP;
		}
		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
}


///--------------------------------------------------------------------------------------
//	SWAnimateSpriteWorld
///--------------------------------------------------------------------------------------

SW_FUNC void SWAnimateSpriteWorld(
	SpriteWorldPtr spriteWorldP)
{
	register SpriteLayerPtr 	curSpriteLayerP;
	register SpritePtr 			curSpriteP;
	SpritePtr 					headActiveSpriteP = NULL;
	SpritePtr 					curActiveSpriteP = NULL;
	Rect						screenDestRect;
	
	if (!spriteWorldP->frameHasOccured)
		return;
	
	
	//-----------------erase the sprites--------------------
	
		// the current port should be the one in which we are drawing    
	SetGWorld(spriteWorldP->workFrameP->framePort, nil);
	
	curSpriteLayerP = spriteWorldP->headSpriteLayerP;	
	
		// iterate through the layers in this world
	while (curSpriteLayerP != NULL)
	{
		curSpriteP = curSpriteLayerP->headSpriteP;
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{
			if ((curSpriteP->needsToBeDrawn && curSpriteP->isVisible) ||
				(curSpriteP->needsToBeErased && !curSpriteP->isVisible))
			{
				if (headActiveSpriteP == NULL)
					headActiveSpriteP = curSpriteP;
				
				if (curActiveSpriteP != NULL)
						curActiveSpriteP->nextActiveSpriteP = curSpriteP;
				
				curActiveSpriteP = curSpriteP;
				
					// union last rect and current rect
					// this way is much faster than UnionRect
				curSpriteP->deltaFrameRect.top =
					SW_MIN(curSpriteP->oldFrameRect.top, curSpriteP->destFrameRect.top);
				curSpriteP->deltaFrameRect.left =
					SW_MIN(curSpriteP->oldFrameRect.left, curSpriteP->destFrameRect.left);
				curSpriteP->deltaFrameRect.bottom =
					SW_MAX(curSpriteP->oldFrameRect.bottom, curSpriteP->destFrameRect.bottom);
				curSpriteP->deltaFrameRect.right =
					SW_MAX(curSpriteP->oldFrameRect.right, curSpriteP->destFrameRect.right);
				{
					short temp;
					
						// align the left edge to long word boundary
					curSpriteP->deltaFrameRect.left &=
							(spriteWorldP->workFrameP->leftAlignFactor);
					
						// align the right edge to long word boundary
					temp = curSpriteP->deltaFrameRect.right &
						spriteWorldP->workFrameP->rightAlignFactor;
					if (temp != 0)
					{
						curSpriteP->deltaFrameRect.right +=
								(spriteWorldP->workFrameP->rightAlignFactor + 1) - temp;
					}
					
						// align the left edge to long word boundary
					curSpriteP->oldFrameRect.left &=
						(spriteWorldP->workFrameP->leftAlignFactor);
					
						// align the right edge to long word boundary
					temp = curSpriteP->oldFrameRect.right &
						spriteWorldP->workFrameP->rightAlignFactor;
					if (temp != 0)
					{
						curSpriteP->oldFrameRect.right +=
							(spriteWorldP->workFrameP->rightAlignFactor + 1) - temp;
					}
				}
				
					// copy the back drop piece
				(*spriteWorldP->offscreenDrawProc)(
						spriteWorldP->backFrameP,
						spriteWorldP->workFrameP,
						&curSpriteP->oldFrameRect,
						&curSpriteP->oldFrameRect);
			}
			
			curSpriteP = curSpriteP->nextSpriteP;
		}
		
		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;	
	}
	
	if (curActiveSpriteP != NULL)
			curActiveSpriteP->nextActiveSpriteP = NULL;
	
	
	//-----------------draw the sprites--------------------
	
	curSpriteLayerP = spriteWorldP->headSpriteLayerP;
	
		// iterate through the layers in this world
	while (curSpriteLayerP != NULL)
	{
		curSpriteP = curSpriteLayerP->headSpriteP;
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{
			if (curSpriteP->isVisible)
			{
				if (curSpriteP->needsToBeDrawn)
				{
						// draw the sprite in the work area
					(*curSpriteP->frameDrawProc)(
							curSpriteP->curFrameP,
							spriteWorldP->workFrameP,
							&curSpriteP->curFrameP->frameRect,
							&curSpriteP->destFrameRect);
					
					if (curSpriteP->isUnderTiles && spriteWorldP->tilingIsOn)
					{
						screenDestRect = curSpriteP->destFrameRect;
						
							// Clip screenDestRect to visScrollRect
						if (screenDestRect.left < spriteWorldP->visScrollRect.left)
							screenDestRect.left = spriteWorldP->visScrollRect.left;
						if (screenDestRect.right > spriteWorldP->visScrollRect.right)
							screenDestRect.right = spriteWorldP->visScrollRect.right;
						if (screenDestRect.top < spriteWorldP->visScrollRect.top)
							screenDestRect.top = spriteWorldP->visScrollRect.top;
						if (screenDestRect.bottom > spriteWorldP->visScrollRect.bottom)
							screenDestRect.bottom = spriteWorldP->visScrollRect.bottom;
		
						SWDrawTilesAboveSprite(spriteWorldP, &screenDestRect);
					}
				}
				else
				{
					SWCheckIdleSpriteOverlap(spriteWorldP, curSpriteP, headActiveSpriteP);
				}
			}
			
			curSpriteP = curSpriteP->nextSpriteP;
		}
		
		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
	
	
	//-----------------update the screen--------------------
	
		// the current port should be the one in which we are drawing    
	SetGWorld(spriteWorldP->windowFrameP->framePort, nil);
	
		// wait for the VBL
	if ( spriteWorldP->usingVBL )
	{  
		spriteWorldP->vblTaskRec.hasVBLFired = false;
		while ( !spriteWorldP->vblTaskRec.hasVBLFired )
		{}
	}
	
	
	curSpriteP = headActiveSpriteP;
	
		// Has a custom worldRect been used? (If so, sprites need offsetting)
	if (spriteWorldP->windRect.left || spriteWorldP->windRect.top)
	{
			// Update on screen the tiles that have changed
		while (spriteWorldP->numTilesChanged > 0)
		{
			screenDestRect = spriteWorldP->changedTiles[--spriteWorldP->numTilesChanged];
			
				// offset dest rect for screen
			screenDestRect.left += spriteWorldP->windRect.left;
			screenDestRect.right += spriteWorldP->windRect.left;
			screenDestRect.top += spriteWorldP->windRect.top;
			screenDestRect.bottom += spriteWorldP->windRect.top;
			
			(*spriteWorldP->screenDrawProc)(
					spriteWorldP->workFrameP,
					spriteWorldP->windowFrameP, 
					&spriteWorldP->changedTiles[spriteWorldP->numTilesChanged], 
					&screenDestRect);
		}
		
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{	
				// offset dest rect for screen
			screenDestRect.left = curSpriteP->deltaFrameRect.left + 
				spriteWorldP->windRect.left;
			screenDestRect.right = curSpriteP->deltaFrameRect.right + 
				spriteWorldP->windRect.left;
			screenDestRect.top = curSpriteP->deltaFrameRect.top + 
				spriteWorldP->windRect.top;
			screenDestRect.bottom = curSpriteP->deltaFrameRect.bottom + 
				spriteWorldP->windRect.top;
			
			(*spriteWorldP->screenDrawProc)(
					spriteWorldP->workFrameP,
					spriteWorldP->windowFrameP,
					&curSpriteP->deltaFrameRect,
					&screenDestRect);
			
			
				// set the delta and last rect to the current rect
			curSpriteP->deltaFrameRect = curSpriteP->destFrameRect;
			curSpriteP->oldFrameRect = curSpriteP->destFrameRect;
			
				// this sprite no longer needs to be drawn
			curSpriteP->needsToBeDrawn = false;
			curSpriteP->needsToBeErased = false;
			
			curSpriteP = curSpriteP->nextActiveSpriteP;
		}
	}
	else
	{
			// Update on screen the tiles that have changed
		while (spriteWorldP->numTilesChanged > 0)
		{
			screenDestRect = spriteWorldP->changedTiles[--spriteWorldP->numTilesChanged];
			
			(*spriteWorldP->screenDrawProc)(
					spriteWorldP->workFrameP,
					spriteWorldP->windowFrameP, 
					&screenDestRect, 
					&screenDestRect);
		}
	
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{	
			(*spriteWorldP->screenDrawProc)(
					spriteWorldP->workFrameP,
					spriteWorldP->windowFrameP,
					&curSpriteP->deltaFrameRect,
					&curSpriteP->deltaFrameRect);
			
			
				// set the delta and last rect to the current rect
			curSpriteP->deltaFrameRect = curSpriteP->destFrameRect;
			curSpriteP->oldFrameRect = curSpriteP->destFrameRect;
			
				// this sprite no longer needs to be drawn
			curSpriteP->needsToBeDrawn = false;
			curSpriteP->needsToBeErased = false;
			
			curSpriteP = curSpriteP->nextActiveSpriteP;
		}
	}
}


///--------------------------------------------------------------------------------------
//	SWCheckIdleSpriteOverlap
///--------------------------------------------------------------------------------------

SW_FUNC void SWCheckIdleSpriteOverlap(
	SpriteWorldPtr			spriteWorldP,
	register SpritePtr		idleSpriteP,
	SpritePtr				headActiveSpriteP)
{
	register SpritePtr 		activeSpriteP = headActiveSpriteP;
	short					srcHorizOffset;
	short					srcVertOffset;
	
	
		// iterate through the active sprites
	while (activeSpriteP != NULL)
	{
			// do the sprites overlap?
		if ((idleSpriteP->oldFrameRect.top < activeSpriteP->deltaFrameRect.bottom) &&
			 (idleSpriteP->oldFrameRect.bottom > activeSpriteP->deltaFrameRect.top) &&
			 (idleSpriteP->oldFrameRect.left < activeSpriteP->deltaFrameRect.right) &&
			 (idleSpriteP->oldFrameRect.right > activeSpriteP->deltaFrameRect.left))
		{
			Rect srcSectRect, dstSectRect;

				// calculate the intersection between the idle sprite's destination
				// rect, and the active sprite's delta rect
			dstSectRect.left =
					SW_MAX(idleSpriteP->destFrameRect.left, activeSpriteP->deltaFrameRect.left);
			dstSectRect.top =
					SW_MAX(idleSpriteP->destFrameRect.top, activeSpriteP->deltaFrameRect.top);
			dstSectRect.right =
					SW_MIN(idleSpriteP->destFrameRect.right, activeSpriteP->deltaFrameRect.right);
			dstSectRect.bottom =
					SW_MIN(idleSpriteP->destFrameRect.bottom, activeSpriteP->deltaFrameRect.bottom);
			
			srcSectRect = dstSectRect;
			
			srcHorizOffset = idleSpriteP->curFrameP->frameRect.left;
			srcVertOffset = idleSpriteP->curFrameP->frameRect.top;
			
			srcSectRect.left -= (idleSpriteP->destFrameRect.left - srcHorizOffset);
			srcSectRect.right -= (idleSpriteP->destFrameRect.left - srcHorizOffset);
			srcSectRect.top -= (idleSpriteP->destFrameRect.top - srcVertOffset);
			srcSectRect.bottom -= (idleSpriteP->destFrameRect.top  - srcVertOffset);
			
			
				// copy a piece of the sprite image onto the back drop piece	
			
			(*idleSpriteP->frameDrawProc)(
					idleSpriteP->curFrameP,
					spriteWorldP->workFrameP,
					&srcSectRect,
					&dstSectRect);
			
			if (idleSpriteP->isUnderTiles && spriteWorldP->tilingIsOn)
			{
					// Clip dstSectRect to visScrollRect
				if (dstSectRect.left < spriteWorldP->visScrollRect.left)
					dstSectRect.left = spriteWorldP->visScrollRect.left;
				if (dstSectRect.right > spriteWorldP->visScrollRect.right)
					dstSectRect.right = spriteWorldP->visScrollRect.right;
				if (dstSectRect.top < spriteWorldP->visScrollRect.top)
					dstSectRect.top = spriteWorldP->visScrollRect.top;
				if (dstSectRect.bottom > spriteWorldP->visScrollRect.bottom)
					dstSectRect.bottom = spriteWorldP->visScrollRect.bottom;

				SWDrawTilesAboveSprite(spriteWorldP, &dstSectRect);
			}
		}
		
		activeSpriteP = activeSpriteP->nextActiveSpriteP;
	}
}


///--------------------------------------------------------------------------------------
//	SWFindSpritesToBeRemoved
///--------------------------------------------------------------------------------------

SW_FUNC void SWFindSpritesToBeRemoved(
	SpriteWorldPtr spriteWorldP )
{
	Boolean						anyRemovableSpritesLeft;
	register SpriteLayerPtr		curSpriteLayerP;
	register SpritePtr			curSpriteP, nextSpriteP;
	
	
	anyRemovableSpritesLeft = false;
	
	curSpriteLayerP = spriteWorldP->headSpriteLayerP;	
	
		// iterate through the layers in this world
	while (curSpriteLayerP != NULL)
	{
		curSpriteP = curSpriteLayerP->headSpriteP;
		
			// iterate through the sprites in this layer
		while (curSpriteP != NULL)
		{	
			nextSpriteP = curSpriteP->nextSpriteP;
			
				// do we want to remove this sprite?
			if (curSpriteP->spriteRemoval != kSWDontRemoveSprite)
			{
					// has this sprite been erased by SWAnimate?
				if (curSpriteP->needsToBeErased == false)
				{
						// if so, we can safely remove it from its layer
					SWRemoveSprite(curSpriteLayerP, curSpriteP);
						// do we want to dispose of this sprite too?
					if (curSpriteP->spriteRemoval == kSWRemoveAndDisposeSprite)
					{
						SWDisposeSprite(curSpriteP);
					}
				}
				else
				{
						// SWAnimate hasn't erased this sprite yet; we're not done
					anyRemovableSpritesLeft = true;
				}
			}
			curSpriteP = nextSpriteP;
		}
		curSpriteLayerP = curSpriteLayerP->nextSpriteLayerP;
	}
	
	spriteWorldP->anySpritesNeedRemoving = anyRemovableSpritesLeft;
}


///--------------------------------------------------------------------------------------
//	SWSetSpriteWorldMaxFPS
///--------------------------------------------------------------------------------------

SW_FUNC void SWSetSpriteWorldMaxFPS(
	SpriteWorldPtr spriteWorldP,
	short framesPerSec)
{
		// is framesPerSec a valid value?
	if (framesPerSec > 0 && framesPerSec < 1000 )
	{
		spriteWorldP->fpsTimeInterval = 1000/framesPerSec;
	}
		// framesPerSec is an "ignore it" value.
	else
	{
		spriteWorldP->fpsTimeInterval = 0;
	}
}


///--------------------------------------------------------------------------------------
// SWSyncSpriteWorldToVBL
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWSyncSpriteWorldToVBL(
	SpriteWorldPtr	spriteWorldP,
	Boolean			syncingOn)
{
	OSErr			err = noErr;
	AuxDCEHandle	mainDCE;
				
	
	if ( syncingOn )
	{
		if ( !spriteWorldP->usingVBL )
		{	
			*((VBLUPP*)&spriteWorldP->vblTaskRec.myVBLTask.vblAddr) =
				NewVBLProc((ProcPtr)SWVBLTask);
			spriteWorldP->vblTaskRec.myVBLTask.qType = vType;
			spriteWorldP->vblTaskRec.myVBLTask.vblCount = 1;
			spriteWorldP->vblTaskRec.hasVBLFired = false;
			
				// insert the task into the VBL queue
			mainDCE = (AuxDCEHandle)GetDCtlEntry((**spriteWorldP->mainSWGDH).gdRefNum);
			err = SlotVInstall( (QElemPtr) &spriteWorldP->vblTaskRec.myVBLTask, 
					(**mainDCE).dCtlSlot );
			if ( err == noErr )
			{
				spriteWorldP->usingVBL = true;
			}
		}
	}
	else
	{
		if ( spriteWorldP->usingVBL )
		{
			mainDCE = (AuxDCEHandle)GetDCtlEntry((**spriteWorldP->mainSWGDH).gdRefNum);
			err = SlotVRemove( (QElemPtr) &spriteWorldP->vblTaskRec.myVBLTask, 
					(**mainDCE).dCtlSlot );
			spriteWorldP->vblTaskRec.hasVBLFired = true;
			spriteWorldP->usingVBL = false;
#if SW_PPC
			DisposeRoutineDescriptor(spriteWorldP->vblTaskRec.myVBLTask.vblAddr);
#endif
		}
	}

	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
// SWVBLTask
///--------------------------------------------------------------------------------------

#if !GENERATINGCFM
extern	VBLTaskRecPtr GetVBLRec(void)
	= 0x2008;	/* MOVE.L	A0,D0 */
#endif

#if GENERATINGCFM
void SWVBLTask(VBLTaskRecPtr vblTaskPtr)
{
	vblTaskPtr->hasVBLFired = true;
	
		// Reset vblCount so that this procedure executes again
	vblTaskPtr->myVBLTask.vblCount = 1;
}
#else
void SWVBLTask( void )
{
	VBLTaskRecPtr vblTaskPtr;
	
	vblTaskPtr = (VBLTaskRecPtr) GetVBLRec();
	vblTaskPtr->hasVBLFired = true;
	
		// Reset vblCount so that this procedure executes again
	vblTaskPtr->myVBLTask.vblCount = 1;
}
#endif

///--------------------------------------------------------------------------------------
//	SWGetSpriteWorldVersion
///--------------------------------------------------------------------------------------

SW_FUNC unsigned long SWGetSpriteWorldVersion(void)
{
	return 0x02000000;
}
